@@ -14,7 +14,7 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: Paint.c,v 1.17 1996/06/19 19:02:29 torsten Exp $ */
+/* $Id: Paint.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -28,12 +28,6 @@
 #include "xpaint.h"
 #include "misc.h"
 
-/*
-**  Value at which zoom starts putting in "white" lines
-**    between pixels
- */
-#define ZOOM_THRESH 5
-
 
 typedef void (*func_t) (Widget, void *, XEvent *, Boolean);
 typedef struct data_s {
@@ -599,7 +593,7 @@ drawVisibleGrid(PaintWidget pw, Widget w, Boolean flag,
 **
  */
 
-static void
+void
 zoomDrawRects(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	      Boolean isExpose, int xstart, int ystart, int zoom,
 	      XRectangle * rect)
@@ -774,14 +768,14 @@ zoomDrawImage(PaintWidget pw, Widget w, GC gc, XImage * xim, XImage * mask,
 	XDestroyImage(dstMsk);
 }
 
-static void
+void
 zoomUpdate(PaintWidget pw, Boolean isExpose, XRectangle * rect)
 {
     int zoom = GET_ZOOM(pw);
     int sx, sy, w, h, zsx, zsy;
-    XImage *xim;
     XRectangle *isec, *tmp;
     XRectangle real, grab;
+    XImage *xim;
 
     if ((isec = RectIntersect(rect, GetRectangle(pw))) == NULL)
 	return;
