@@ -15,7 +15,7 @@
 
 /* Portions copyright 1995, 1995 Torsten Martinsen */
 
-/* $Id: PaintRegion.c,v 1.7 1998/01/22 20:55:31 torsten Exp $ */
+/* $Id: PaintRegion.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 /*
 ** PaintRegion.c -- Hopefully all of the routines to get, set and  
@@ -26,6 +26,7 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <math.h>
 #ifdef __EMX__
 #include <float.h>
@@ -34,7 +35,10 @@
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
 #include <X11/Xatom.h>
-#include <X11/Xaw/Grip.h>
+
+#include "xaw_incdir/Grip.h"
+
+#include "xpaint.h"
 #include "PaintP.h"
 #include "protocol.h"
 
@@ -100,8 +104,6 @@ static void regionExpose(Widget, PaintWidget, XEvent *, Boolean *);
 		mm(pw->paint.region.rotMat, m, pw->paint.region.mat);	\
 	} while (0)
 
-static int transparent = 0;
-
 static pwMatrix matIdentity =
 {
     {1, 0},
@@ -167,22 +169,23 @@ buildSources(PaintWidget pw)
 }
 
 void 
-RegionTransparency(PaintWidget pw, int mode)
+RegionTransparency(PaintWidget pw)
 {
     Widget w;
-    XImage *src1, *src2;
-    Pixel pix1, pix2;
-    XRectangle rect;
-    int rwidth, rheight, pwidth, pheight;
+    XImage *src1, *src2, *src3;
+    Pixel pix;
+    int rwidth, rheight, zoom, skip;
     int px, py, rx, ry, x, y, i, j, bytes_per_pixel;
+    char bg[4];
 
+    zoom = GET_ZOOM(pw);
+    /* if (zoom!=1) return; */ /* don't avoid it now ! */
+   
     w = pw->paint.region.child;
     if (!pw->paint.region.source || !w) {
-       transparent = 0;
        return;
     }
 
-    transparent = !mode;
     px = pw->paint.region.rect.x;
     py = pw->paint.region.rect.y;
     if (px<0) {
@@ -208,39 +211,76 @@ RegionTransparency(PaintWidget pw, int mode)
     if (rwidth<=0 || rheight<=0)
        return;
 
+    rwidth /= zoom;
+    rheight /= zoom;   
+   
+    /* printf("%d %d %d %d %d %d\n", px, py, rx, ry, rwidth, rheight); */
     src1 = XGetImage(XtDisplay(pw), pw->paint.region.source,
-       	             rx, ry, rwidth, rheight,
+       	             (zoom==1)?rx:0, (zoom==1)?ry:0, rwidth, rheight,
 		     AllPlanes, ZPixmap);
-    if (mode) {
-       XPutImage(XtDisplay(pw), XtWindow(w), 
-          pw->paint.region.fg_gc == None ?
-	     pw->paint.tgc : pw->paint.region.fg_gc,
-          src1, 0, 0, rx, ry, rwidth, rheight);
-       XDestroyImage(src1);
-       return;
-    }
 
-    src2 = XGetImage(XtDisplay(pw), GET_PIXMAP(pw),
+   if (!Global.transparent) {
+       src2 = src1;
+       bytes_per_pixel = src2->bits_per_pixel/8;
+   } else {
+       src2 = XGetImage(XtDisplay(pw), GET_PIXMAP(pw),
 		     px, py, rwidth, rheight,
 		     AllPlanes, ZPixmap);
-    bytes_per_pixel = src1->bits_per_pixel/8;
-
-    for (y=0; y<rheight; y++)
-       for (x=0; x<rwidth; x++) if ((x+y)&1) {
-	  i = bytes_per_pixel * x + src1->bytes_per_line * y;
-	  for (j=0; j<bytes_per_pixel; j++) {
-	     src1->data[i] = src2->data[i];
-	     ++i;
-       	  }
+       bytes_per_pixel = src2->bits_per_pixel/8;
+       for (y=0; y<rheight; y++) {
+           i = src2->bytes_per_line * y;
+           for (x=0; x<rwidth; x++) {
+	       if ((x+y)&1)
+	           memcpy(&src2->data[i], &src1->data[i], bytes_per_pixel);
+	       i += bytes_per_pixel;
+           }
        }
+       XDestroyImage(src1);
+   }
 
-    XPutImage(XtDisplay(pw), XtWindow(w), 
-       pw->paint.region.fg_gc == None ?
+   if (zoom == 1) {
+       XPutImage(XtDisplay(pw), XtWindow(w), 
+          pw->paint.region.fg_gc == None ?
 	  pw->paint.tgc : pw->paint.region.fg_gc,
-       src1, 0, 0, rx, ry, rwidth, rheight);
-
-    XDestroyImage(src1);
-    XDestroyImage(src2);
+          src2, 0, 0, rx, ry, rwidth, rheight);
+       XDestroyImage(src2);
+    } else {
+       rwidth *= zoom;
+       rheight *= zoom;
+       src3 = XCreateImage(XtDisplay(pw), pw->paint.visual,
+                        src2->depth, ZPixmap, 0, NULL,
+		        rwidth, rheight,
+		        32, 0);
+       src3->data = (char *) XtMalloc(rheight * src3->bytes_per_line);
+       XtVaGetValues((Widget)pw, XtNbackground, &pix, NULL);
+       XPutPixel(src3, 0, 0, pix);
+       memcpy(bg, src3->data, bytes_per_pixel);
+       skip = -(zoom<=ZOOM_THRESH);
+       for (y=0; y<rheight; y++) {
+           i = src2->bytes_per_line * (y/zoom);
+           j = src3->bytes_per_line * y;
+	   if ((y+1)%zoom == skip)
+           for (x=0; x<rwidth; x++) {
+	       memcpy(&src3->data[j], bg, bytes_per_pixel);
+	       j += bytes_per_pixel;
+	   } else
+           for (x=0; x<rwidth; x++) {
+	       if ((x+1)%zoom == skip)
+		  memcpy(&src3->data[j], bg, bytes_per_pixel);
+               else
+	          memcpy(&src3->data[j], &src2->data[i], bytes_per_pixel);
+	       if ((x+1)%zoom == 0)
+	           i += bytes_per_pixel;
+	       j += bytes_per_pixel;
+	   }
+       }
+       XDestroyImage(src2);
+       XPutImage(XtDisplay(pw), XtWindow(w), 
+           pw->paint.region.fg_gc == None ?
+	   pw->paint.tgc : pw->paint.region.fg_gc,
+           src3, 0, 0, rx, ry, rwidth, rheight);
+       XDestroyImage(src3);
+    }
 }
 
 static void 
@@ -773,6 +813,7 @@ static void
 regionButtonPress(Widget w, PaintWidget pw, XButtonEvent * event, Boolean * junk)
 {
     int zoom = GET_ZOOM(pw);
+    static long prev_time = 0;
 
     pw->paint.region.isRotate = (event->button == Button2);
 
@@ -780,6 +821,19 @@ regionButtonPress(Widget w, PaintWidget pw, XButtonEvent * event, Boolean * junk
     pw->paint.region.offY = event->y;
     pw->paint.region.baseX = event->x_root - w->core.x;
     pw->paint.region.baseY = event->y_root - w->core.y;
+   
+    if (event->button == Button1) {
+        if (abs(event->time-prev_time) > 300) {
+            Global.transparent = !Global.transparent;
+            PwRegionTear((Widget)pw);
+            RegionTransparency(pw);
+	} else {
+            prev_time = event->time;
+	    return;
+	}
+        prev_time = event->time;
+    }
+      
     motionExtern((Widget)pw, (XEvent *) event,
 	pw->paint.region.rect.x, 
         pw->paint.region.rect.y + pw->paint.region.rect.height/zoom - 1, 1);
@@ -813,6 +867,13 @@ static void
 regionButtonRelease(Widget w, PaintWidget pw, XButtonEvent * event,
 		    Boolean * junk)
 {
+
+    if (event->button == Button1) {
+        Global.transparent = !Global.transparent;
+        PwRegionTear((Widget)pw);
+        RegionTransparency(pw);
+    }
+      
     pw->paint.region.isTracking = False;
     drawRegionBox(pw, False);
 
@@ -893,13 +954,7 @@ regionGrab(Widget w, PaintWidget pw, XMotionEvent * event, Boolean * junk)
         motionExtern((Widget)pw, (XEvent *) event,
 	   pw->paint.region.rect.x, 
            pw->paint.region.rect.y + pw->paint.region.rect.height/zoom - 1, 0);
-        /* 0xF8 is Mod?Mask */
-	if ((zoom==1) && (event->state & 0xF8))
-           RegionTransparency(pw, 0);
-	else {
-	   if (zoom==1 && transparent)
-	      RegionTransparency(pw, 1);
-	}
+        RegionTransparency(pw);
     }
 }
 
@@ -1442,6 +1497,7 @@ PwRegionSet(Widget w, XRectangle * rect, Pixmap pix, Pixmap mask)
 	return;
 
     pw->paint.region.curZoom = zoom;
+    Global.transparent = 0;
 
     x = (rect->x + pw->paint.zoomX) * zoom;
     y = (rect->y + pw->paint.zoomY) * zoom;
