@@ -12,7 +12,7 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: arcOp.c,v 1.3 1996/04/19 09:04:59 torsten Exp $ */
+/* $Id: arcOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <math.h>
 #include <stdlib.h>
@@ -39,11 +39,13 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     /*
     **	Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;
     if ((event->state & AllButtonsMask) != 0)
 	return;
     if (event->button == Button3) return;
 
     if (l->tracking == -1) {
+        Global.escape = 0;
         l->mode = arcMode;
         l->endX = l->midX = l->startX = event->x;
         l->endY = l->midY = l->startY = event->y;
@@ -300,6 +302,7 @@ release(Widget w, LocalInfo * l,
     /*
     **	Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;   
     mask = AllButtonsMask;
     switch (event->button) {
     case Button1:
@@ -325,8 +328,13 @@ release(Widget w, LocalInfo * l,
     if (l->tracking == -1) return;
 
     if (info->surface == opWindow) {
-	DrawArc(w, info->drawable, l->gcx, l);
 
+	if (l->mode == 1 && l->tracking == 0) {
+	   l->tracking = 1;
+	   return;
+	}
+        DrawArc(w, info->drawable, l->gcx, l);
+       
         if (l->tracking == 0) {
             l->midX = l->endX = event->x;
             l->midY = l->endY = event->y;
@@ -346,12 +354,21 @@ release(Widget w, LocalInfo * l,
 	        l->tracking = -1;
 	}
 	if (l->tracking == 10) {
+	    if (Global.escape) {
+	       l->tracking = -1;
+               return;
+	    }
             SetCapAndJoin(w, l->lgc, JoinRound);
 	    if (!info->isFat)
                 DrawArc(w, info->drawable, l->lgc, l);
 	}
     } else {
         if (l->tracking == 10) {
+	    if (Global.escape) {
+	       Global.escape = 0;
+	       l->tracking = -1;
+               return;
+	    }
 	    l->tracking = 20;
             SetCapAndJoin(w, l->lgc, JoinRound);
 	    XtVaGetValues(w, XtNzoom, &l->zoom, NULL);
