@@ -13,16 +13,20 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: brushOp.c,v 1.18 1998/04/16 19:38:10 torsten Exp $ */
+/* $Id: brushOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <stdlib.h>
+#include <math.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Box.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Toggle.h>
-#include <stdlib.h>
+
+#include "xaw_incdir/Box.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Toggle.h"
+
 #include <xpm.h>
 
 #include "xpaint.h"
@@ -36,6 +40,7 @@
 #include "ops.h"
 #include "rc.h"
 
+
 #include "bitmaps/brushes/paintA.xpm"
 #include "bitmaps/brushes/paintB.xpm"
 #include "bitmaps/brushes/paintC.xpm"
@@ -57,6 +62,8 @@
 #include "bitmaps/brushes/paintS.xpm"
 #include "bitmaps/brushes/paintT.xpm"
 
+enum { OPAQUE, TRANSPARENT, STAIN };
+
 #define BRUSH(name) (char **) CONCAT(name, _xpm)
 
 /* this blend function ensures to reach the final color - ACZ: */
@@ -101,6 +108,10 @@ typedef enum {
     ERASE, SMEAR, PLAIN
 } BrushType;
 
+typedef struct {
+    Widget form, box, close;   
+} BrushboxInfo;
+
 typedef struct {
     int useSecond;
     BrushType brushtype;
@@ -113,10 +124,14 @@ typedef struct {
 
 static BrushItem *currentBrush = NULL;
 static Boolean eraseMode = True;
-static Boolean transparentMode = False;
+static int transparentMode = 0;
 static float brushOpacity = 0.2;
 static XImage *brushImage;
 
+/* RPC */
+static void stain(Widget w, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy);
+/* RPC */
+
 static void smear(Widget w, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy);
 static void wbrush(Widget w, OpInfo * info, GC gc,
 		   LocalInfo * l, int sx, int sy);
@@ -141,8 +156,12 @@ draw(Widget w, OpInfo * info, LocalInfo * l, int x, int y)
 		  gc, sx, sy, l->width, l->height, sx, sy);
     } else if (l->brushtype == SMEAR) {
 	smear(w, info, gc, l, sx, sy);
-    } else if (transparentMode) {
+    } else if (transparentMode==TRANSPARENT) {
 	wbrush(w, info, gc, l, sx, sy);
+/* RPC */
+    } else if (transparentMode==STAIN) {
+	stain(w, info, gc, l, sx, sy );
+/* RPC */
     } else {			/* plain opaque brush */
 	XFillRectangle(XtDisplay(w), info->drawable,
 		       gc, sx, sy, l->width, l->height);
@@ -162,6 +181,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     /*
     **  Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;   
     if ((event->state & (Button1Mask | Button2Mask | Button3Mask |
 			 Button4Mask | Button5Mask)) != 0)
 	return;
@@ -241,7 +261,8 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     /*
     **  Check to make sure all buttons are up, before doing this
      */
-    mask = AllButtonsMask;
+    if (event->button >= Button4) return;   
+    mask = AllButtonsMask;   
     switch (event->button) {
     case Button1:
 	mask ^= Button1Mask;
@@ -270,6 +291,175 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     XSetClipMask(XtDisplay(w), info->base_gc, None);
 }
 
+/* RPC */
+
+static void 
+rgbTOhsv(float r, float g, float b, float *h, float *s, float *v)
+{
+    float max = MAX(r, MAX(g, b));
+    float min = MIN(r, MIN(g, b));
+    float delta;
+
+    *v = max;
+    if (max != 0.0)
+	*s = (max - min) / max;
+    else
+	*s = 0.0;
+
+    if (*s == 0.0) {
+	*h = 0.0;
+    } else {
+	delta = max - min;
+	if (r == max)
+	    *h = (g - b) / delta;
+	else if (g == max)
+	    *h = 2.0 + (b - r) / delta;
+	else			/* if (b == max) */
+	    *h = 4.0 + (r - g) / delta;
+	*h *= 60.0;
+	if (*h < 0.0)
+	    *h += 360.0;
+    }
+}
+
+static void 
+hsvTOrgb(float h, float s, float v, float *r, float *g, float *b)
+{
+    int i;
+    float f, p, q, t;
+
+    if (s == 0 && h == 0) {
+	*r = *g = *b = v;
+    } else {
+	if (h >= 360.0)
+	    h = 0.0;
+	h /= 60.0;
+
+	i = h;
+	f = h - i;
+	p = v * (1 - s);
+	q = v * (1 - (s * f));
+	t = v * (1 - (s * (1 - f)));
+	switch (i) {
+	case 0:
+	    *r = v;
+	    *g = t;
+	    *b = p;
+	    break;
+	case 1:
+	    *r = q;
+	    *g = v;
+	    *b = p;
+	    break;
+	case 2:
+	    *r = p;
+	    *g = v;
+	    *b = t;
+	    break;
+	case 3:
+	    *r = p;
+	    *g = q;
+	    *b = v;
+	    break;
+	case 4:
+	    *r = t;
+	    *g = p;
+	    *b = v;
+	    break;
+	case 5:
+	    *r = v;
+	    *g = p;
+	    *b = q;
+	    break;
+	}
+    }
+}
+/*
+** drawing routine for stain operator
+ */
+static void
+stain(Widget wid, OpInfo * info, GC gc, LocalInfo * l, int sx, int sy)
+{
+    int x, y, dx, dy, w, h, d;
+    unsigned char *brushbits;
+    Display *dpy = XtDisplay(wid);
+    XGCValues gcValues;
+    XColor *brushCol, *col;
+    Pixel p;
+    float r, g, b, H, S, V;
+    float brushH, brushS, brushV;
+
+    /*
+     * Perform manual clipping to avoid XPutImage crashing on us
+     */
+
+    XGetGCValues( dpy, gc, GCForeground, &gcValues );
+
+    brushCol = PaletteLookup(l->brushPalette, gcValues.foreground);
+
+    r = brushCol->red / 65535.0;
+    g = brushCol->green / 65535.0;
+    b = brushCol->blue / 65535.0;
+
+    rgbTOhsv( r, g, b, &brushH, &brushS, &brushV );
+
+    dx = dy = 0;
+    w = l->width;
+    h = l->height;
+
+    if (sx < 0) {
+	dx = -sx;
+	w += sx;
+	sx = 0;
+    }
+    if ((d = (sx + w - ((PaintWidget) wid)->paint.drawWidth)) > 0)
+	w -= d;
+    if (sy < 0) {
+	dy = -sy;
+	h += sy;
+	sy = 0;
+    }
+    if ((d = (sy + h - ((PaintWidget) wid)->paint.drawHeight)) > 0)
+	h -= d;
+    if ((w <= 0) || (h <= 0))
+	return;
+
+    /* copy portion of image under brush into brushImage */
+    XGetSubImage(dpy, info->drawable, sx, sy, w, h,
+		 AllPlanes, ZPixmap, brushImage, dx, dy);
+
+    brushbits = (unsigned char *) currentBrush->brushbits;
+
+    for (y = 0; y < l->height; ++y)
+	for (x = 0; x < l->width; ++x)
+	    if (*brushbits++) {
+		p = XGetPixel(brushImage, x, y);
+		col = PaletteLookup(l->brushPalette, p);
+
+    		r = col->red;
+    		g = col->green;
+    		b = col->blue;
+
+		rgbTOhsv( r, g, b, &H, &S, &V );
+
+		H = brushH;
+		S = brushS;
+
+		hsvTOrgb( H, S, V, &r, &g, &b );
+
+		col->red = (unsigned short int) (r);
+		col->green = (unsigned short int) (g);
+		col->blue = (unsigned short int) (b);
+
+		p = PaletteAlloc(l->brushPalette, col);
+		XPutPixel(brushImage, x, y, p );
+	    }
+
+    XPutImage(dpy, info->drawable, gc, brushImage, dx, dy, sx, sy, w, h);
+}
+
+/* RPC */
+
 /*
 ** drawing routine for smear operator
  */
@@ -552,7 +742,7 @@ EraseSetMode(Boolean mode)
 }
 
 void
-BrushSetMode(Boolean mode)
+BrushSetMode(int mode)
 {
     transparentMode = mode;
 }
@@ -744,10 +934,32 @@ selectBrush(Widget shell, BrushItem * nc)
 			   currentBrush->width, currentBrush->height);
 }
 
+static void
+brushboxResized(Widget w, BrushboxInfo * l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height;
+    XtVaGetValues(w, XtNwidth, &width,
+                     XtNheight, &height, NULL);
+    if (width<10) width = 10;
+    if (height<35) height = 35;
+    XtResizeWidget(l->box, width-6, height-32,
+#ifdef XAW3D		   
+		   1
+#else
+		   0
+#endif		   
+		   );
+    XtUnmanageChild(l->close);
+    XMapWindow(XtDisplay(l->close), XtWindow(l->close));
+    XtMoveWidget(l->close, 4, height-25);
+}
+
+
 static Widget
 createBrushDialog(Widget w)
 {
-    Widget shell, form, box, icon, firstIcon = 0, close;
+    Widget shell, icon, firstIcon = 0;
+    static BrushboxInfo *info = NULL;
     GC gc;
     XGCValues values;
     RCInfo *rcInfo;
@@ -758,19 +970,20 @@ createBrushDialog(Widget w)
     Pixmap pix;
     int nw, nh, ox, oy;
     XpmAttributes xpmAttr;
-    Arg args[3];
+    Arg args[4];
     int nargs = 0;
     
+    if (!info) info = (BrushboxInfo *)XtMalloc(sizeof(BrushboxInfo));
     shell = XtVisCreatePopupShell("brush",
 				 topLevelShellWidgetClass, w,
 				 args, nargs);
 
-    form = XtVaCreateManagedWidget(NULL,
+    info->form = XtVaCreateManagedWidget(NULL,
 				   formWidgetClass, shell,
 				   NULL);
 
-    box = XtVaCreateManagedWidget("box",
-				  boxWidgetClass, form,
+    info->box = XtVaCreateManagedWidget("box",
+				  boxWidgetClass, info->form,
 				  NULL);
 
     values.foreground = WhitePixelOfScreen(XtScreen(w));
@@ -821,10 +1034,10 @@ createBrushDialog(Widget w)
 	    brushList[i].width = brush->width;
             brushList[i].height = brush->height;
 	    brushList[i].pixmap = XCreatePixmap(dpy,
-		RootWindowOfScreen(XtScreen(box)),
+		RootWindowOfScreen(XtScreen(info->box)),
 	        brushList[i].width, brushList[i].height, 1);
 	    scale = brush->scale;
-	    ptr = brush->data;
+	    ptr = (char*) brush->data;
 	    gc0 = XCreateGC(dpy, brushList[i].pixmap, 
                             GCForeground | GCBackground, &values);
 	    gc1 = XCreateGC(dpy, brushList[i].pixmap, 
@@ -853,16 +1066,19 @@ createBrushDialog(Widget w)
 	    xpmAttr.colorsymbols = monoColorSymbols;
 	    xpmAttr.numsymbols = 5;
 	    xpmAttr.valuemask = XpmDepth | XpmColorSymbols;
-	    if (XpmCreatePixmapFromData(XtDisplay(box),
-				    RootWindowOfScreen(XtScreen(box)),
+	    if (XpmCreatePixmapFromData(XtDisplay(info->box),
+				    RootWindowOfScreen(XtScreen(info->box)),
 				 brushList[i].bits, &brushList[i].pixmap,
 				    NULL, &xpmAttr)) continue;
 	    brushList[i].width = xpmAttr.width;
 	    brushList[i].height = xpmAttr.height;
 	}
 	icon = XtVaCreateManagedWidget("icon",
-				       toggleWidgetClass, box,
+				       toggleWidgetClass, info->box,
 				       XtNradioGroup, firstIcon,
+#ifdef XAW3D
+				       XtNforeground, 0xc0c0c0,
+#endif				       
 				       NULL);
 	nw = brushList[i].width;
 	nh = brushList[i].height;
@@ -875,13 +1091,13 @@ createBrushDialog(Widget w)
 	    oy = (16 - nh) / 2;
 	    nh = 16;
 	}
-	pix = XCreatePixmap(XtDisplay(box),
-			    RootWindowOfScreen(XtScreen(box)),
+	pix = XCreatePixmap(XtDisplay(info->box),
+			    RootWindowOfScreen(XtScreen(info->box)),
 			    nw, nh,
-			    DefaultDepthOfScreen(XtScreen(box)));
+			    DefaultDepthOfScreen(XtScreen(info->box)));
 
-	XtVaGetValues(icon, XtNforeground, &fg, XtNbackground, &bg,
-		      NULL);
+	XtVaGetValues(icon, XtNbackground, &bg, NULL);
+        fg = XtScreen(w)->black_pixel;
 	/*
 	**  Clear then draw the clipped rectangle in
 	 */
@@ -903,18 +1119,20 @@ createBrushDialog(Widget w)
 	       (XtCallbackProc) selectBrush, (XtPointer) & brushList[i]);
     }
 
-    close = XtVaCreateManagedWidget("close",
-				    commandWidgetClass, form,
-				    XtNfromVert, box,
+    info->close = XtVaCreateManagedWidget("close",
+				    commandWidgetClass, info->form,
+				    XtNfromVert, info->box,
 				    XtNtop, XtChainBottom,
 				    NULL);
 
-    XtAddCallback(close, XtNcallback, (XtCallbackProc) closePopup,
+    XtAddCallback(info->close, XtNcallback, (XtCallbackProc) closePopup,
 		  (XtPointer) shell);
 
     XFreeGC(XtDisplay(w), gc);
 
     AddDestroyCallback(shell, (DestroyCallbackFunc) closePopup, shell);
+    XtAddEventHandler(shell, StructureNotifyMask, False,
+		      (XtEventHandler) brushboxResized, (XtPointer) info);
 
     return shell;
 }
