@@ -14,19 +14,22 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: chroma.c,v 1.8 2000/09/02 08:26:20 torsten Exp $ */
+/* $Id: chroma.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Toggle.h>
+
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Toggle.h"
 #undef NeedFunctionPrototypes
 #define NeedFunctionPrototypes 1
 #undef NeedWidePrototypes
-#include <X11/Xaw/Scrollbar.h>
-#include <X11/Xaw/Command.h>
+#include "xaw_incdir/Scrollbar.h"
+#include "xaw_incdir/Command.h"
+
+
 #include <X11/Shell.h>
 #include <X11/keysym.h>
 #include <stdio.h>
@@ -170,7 +173,7 @@ addSB(Widget parent, Widget above, char *title, Widget * bar, Widget * text)
 				    NULL);
     *bar = XtVaCreateManagedWidget("varianceBar", scrollbarWidgetClass, form,
 				   XtNorientation, XtorientHorizontal,
-#ifdef XAW95
+#if defined(XAW3D) || defined(XAW95)
 				   XtNwidth, 236,
 #else
 				   XtNwidth, 192,
@@ -264,8 +267,7 @@ applyCB(Widget w, DialogInfo * l, XtPointer junk)
 
 #if RANGEBUTTONS
     t = ((int) XawToggleGetCurrent(l->mode)) - 1;
-    OperationSelectCallAcross(t);
-    SelectSetCutMode(t);
+    OperationSelectCall(t);
 #endif
 }
 
@@ -424,6 +426,21 @@ ChromaDialog(Widget w, Palette * map)
         XtUnmanageChild(cpick);
         XMapWindow(XtDisplay(cpick), XtWindow(cpick));
     }
+    XtUnmanageChild(above);      
+    XtUnmanageChild(l->redBar);   
+    XtUnmanageChild(l->greenBar);
+    XtUnmanageChild(l->blueBar);      
+    XtUnmanageChild(apply);
+    XtUnmanageChild(cancel);
+    XMapWindow(XtDisplay(above), XtWindow(above));
+    XMapWindow(XtDisplay(l->redBar), XtWindow(l->redBar));
+    XMapWindow(XtDisplay(l->greenBar), XtWindow(l->greenBar));
+    XMapWindow(XtDisplay(l->blueBar), XtWindow(l->blueBar));
+    XMapWindow(XtDisplay(apply), XtWindow(apply));
+    XMapWindow(XtDisplay(cancel), XtWindow(cancel));
+   
+    XtUnmanageChild(form);         
+    XMapWindow(XtDisplay(form), XtWindow(form));   
 }
 
 #if RANGEBUTTONS
