@@ -13,22 +13,25 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: color.c,v 1.12 2000/09/02 08:26:20 torsten Exp $ */
+/* $Id: color.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <stdio.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Label.h>
+#include <X11/StringDefs.h>
+#include <X11/cursorfont.h>
+
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Label.h"
+
+
 #undef NeedFunctionPrototypes
 #define NeedFunctionPrototypes 1
 #undef NeedWidePrototypes
-#include <X11/Xaw/Scrollbar.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Text.h>
-#include <X11/StringDefs.h>
-#include <X11/cursorfont.h>
+#include "xaw_incdir/Scrollbar.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Text.h"
 
 #ifndef NOSTDHDRS
 #include <stdlib.h>
@@ -265,7 +268,7 @@ update(LocalInfo * l, Boolean doSB, Boolean doText, Boolean doCursor)
 
 #if 0
     if ((l->map != NULL) &&
-	(l->map->readonly ||
+	(l->map->readonly_ ||
 	 ((DefaultVisualOfScreen(XtScreen(l->view))->class & 1) == 0)))
 #endif
 	XtVaSetValues(l->view, XtNbackground,
@@ -816,7 +819,7 @@ ColorPickerShell(Widget w)
     Widget top = GetToplevel(w);
     Widget shell;
     int nargs = 0;
-    Arg args[3];
+    Arg args[4];
 
     shell = XtVisCreatePopupShell("color", transientShellWidgetClass,
 				  GetShell(w), args, nargs);
