@@ -13,13 +13,15 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: colorEdit.c,v 1.4 1997/09/03 19:31:07 torsten Exp $ */
+/* $Id: colorEdit.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Command.h>
+
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Command.h"
+
 #include "misc.h"
 #include "palette.h"
 #include "color.h"
@@ -75,7 +77,7 @@ ColorEditor(Widget w, Pixel pixel, Palette * map, Boolean allowWrite,
     Widget shell, form, ok, cancel, pick;
     Pixel pix;
     LocalInfo *info = XtNew(LocalInfo);
-    Arg args[6];
+    Arg args[8];
     int nargs = 0;
     
     info->okProc = okProc;
@@ -141,4 +143,8 @@ ColorEditor(Widget w, Pixel pixel, Palette * map, Boolean allowWrite,
         XtUnmanageChild(pick);
         XMapWindow(XtDisplay(pick), XtWindow(pick));
     }
+    XtUnmanageChild(ok);   
+    XtUnmanageChild(cancel);
+    XMapWindow(XtDisplay(ok), XtWindow(ok));   
+    XMapWindow(XtDisplay(cancel), XtWindow(cancel));      
 }
