@@ -16,6 +16,11 @@
 /* Drawing with a dynamic pen. Adapted from Paul Haeberli's
    dynadraw.c (for Iris GL),
    see http://www.sgi.com/grafica/dyna/index.html */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtWindow XTWINDOW
+#endif
    
 #include <math.h>
 #include <sys/param.h>
@@ -43,7 +48,7 @@ typedef struct {
 	float	width;
 	float	odelx, odely;
 	float	mass, drag;
-	int	doFinish, finishing;
+	int	doFinish, pressed, finishing;
 	XtIntervalId tid;
 	Widget  w;
 	OpInfo	*opinfo;
@@ -130,13 +135,15 @@ press(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
 
 	if ((event->state & (Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)) != 0)
 		return;
-        if (event->button == Button3) return;
+        if (event->button >= Button3) return;
+        if (event->type == MotionNotify) return;
 
 	l->mass	    = mass;
 	l->drag     = drag;
 	l->width    = width;
 	l->doFinish = doFinish;
 	l->finishing = False;
+	l->pressed = True;
 	l->fixedangle = 0;
 	l->tid	    = 0;
 	l->w	    = w;
@@ -166,9 +173,9 @@ press(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
 static Bool
 dummy_pred(Display *d, XEvent *ev, char *arg)
 {
-	return (ev->type == MotionNotify
-		|| (ev->type == PropertyNotify
-		    && ev->xproperty.atom == dummy_prop));
+	return (ev->type == MotionNotify ||
+		  (ev->type == PropertyNotify
+		     && ev->xproperty.atom == dummy_prop));
 }
 
 static void motion(Widget, LocalInfo *, XMotionEvent *, OpInfo *);
@@ -188,10 +195,10 @@ timeout(XtPointer client_data, XtIntervalId *iid)
 	   or a MotionNotify */
 
 	XPeekIfEvent(XtDisplay(l->w), &ev, dummy_pred, NULL);
-	
-	if (ev.type == MotionNotify)
-		return;
 
+	if (ev.type == MotionNotify)
+	   return;
+   
 	/* It's a PropertyNotify */
 	l->tid = 0;
 	me.x = l->mousex;
@@ -206,12 +213,16 @@ motion(Widget w, LocalInfo *l, XMotionEvent *event, OpInfo *info)
 {
 	XRectangle	undo;
 
+	if (!l->pressed)
+		return;
 	if (info->surface == opWindow)
 		return;
         if (l->gc != info->first_gc && l->gc != info->second_gc) return;
 
-	if (l->tid)
-		XtRemoveTimeOut(l->tid), l->tid = 0;
+	if (l->tid) {
+		XtRemoveTimeOut(l->tid); 
+                l->tid = 0;
+	}
 
 #if 0
 printf("motion: %d: (%d, %d)\n", event->time, event->x, event->y);
@@ -264,7 +275,7 @@ printf("motion: %d: (%d, %d)\n", event->time, event->x, event->y);
 		l->starty = event->y;
 
 		PwUpdate(w, &undo, False);
-
+	     
 		if (!l->finishing)
 			l->tid = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
 						 XTIMEOUT, timeout, l);
@@ -280,6 +291,8 @@ release(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
         /*
         **  Check to make sure all buttons are up, before doing this
         */
+        if (!l->pressed) return;
+   
         mask = Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask;
         switch (event->button) {
         case Button1:   mask ^= Button1Mask; break;
@@ -288,15 +301,24 @@ release(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
         case Button4:   mask ^= Button4Mask; break;
         case Button5:   mask ^= Button5Mask; break;
         }
+   
+        if ((event->type & PointerMotionMask) == MotionNotify) return;
+
         if ((event->state & mask) != 0)
                 return;
-        if (event->button == Button3) return;
 
-	if (l->tid)
-		XtRemoveTimeOut(l->tid), l->tid = 0;
+        if (event->button >= Button3) return;
 
-	if (!l->doFinish)
+	if (l->tid) {
+	    XtRemoveTimeOut(l->tid);
+            l->tid = 0;
+	}
+
+	if (!l->doFinish) {
+                l->pressed = False;
 		return;
+	}
+
 	l->finishing = True;
 
 	me.x = event->x;
@@ -306,6 +328,7 @@ release(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
 		me.time += XTIMEOUT;
 		motion(w, l, &me, info);
 	}
+        l->pressed = False;
 }
 
 /*
@@ -341,8 +364,7 @@ void
 
 	OpAddEventHandler(w, opPixmap, ButtonPressMask, FALSE, (OpEventProc)press, l);
 	OpAddEventHandler(w, opPixmap, ButtonMotionMask, FALSE, (OpEventProc)motion, l);
-	OpAddEventHandler(w, opPixmap, ButtonReleaseMask, FALSE, (OpEventProc)release, l);
-
+	OpAddEventHandler(w, opPixmap, PointerMotionMask|ButtonReleaseMask, FALSE, (OpEventProc)release, l);
 	SetPencilCursor(w);
 
 	if (dummy_prop == None)
@@ -352,6 +374,7 @@ void
 	XSelectInput(XtDisplay(w), XtWindow(w),
 		     wa.your_event_mask | PropertyChangeMask);
 
+        l->pressed = 0;
 	return l;
 }
 
@@ -360,6 +383,6 @@ DynPencilRemove(Widget w, LocalInfo *l)
 {
 	OpRemoveEventHandler(w, opPixmap, ButtonPressMask, FALSE, (OpEventProc)press, l);
 	OpRemoveEventHandler(w, opPixmap, ButtonMotionMask, FALSE, (OpEventProc)motion, l);
-	OpRemoveEventHandler(w, opPixmap, ButtonReleaseMask, FALSE, (OpEventProc)release, l);
+	OpRemoveEventHandler(w, opPixmap, PointerMotionMask|ButtonReleaseMask, FALSE, (OpEventProc)release, l);
 	XtFree((XtPointer)l);
 }
