@@ -13,20 +13,23 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: fatBitsEdit.c,v 1.17 1997/09/03 19:31:07 torsten Exp $ */
+/* $Id: fatBitsEdit.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <stdio.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/IntrinsicP.h>	/* for XtResizeWidget() */
 #include <X11/StringDefs.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Box.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Scrollbar.h>
 #include <X11/Shell.h>
 #include <X11/cursorfont.h>
-#include <stdio.h>
+
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Viewport.h"
+#include "xaw_incdir/Box.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Scrollbar.h"
+
 #ifndef NOSTDHDRS
 #include <stdlib.h>
 #include <unistd.h>
@@ -583,26 +586,29 @@ sizeChanged(Widget w, XtPointer l, XtPointer junk)
 **  The parent box widget changed size, resize to fit.
  */
 static void 
-boxChanged(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
+fatbitsResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
 {
-    Dimension width, height;
-    Dimension hpad, vpad, bw;
+    Dimension width, height, bw;
     int zoom;
 
     XtVaGetValues(l->view, XtNzoom, &zoom, XtNborderWidth, &bw,
 		  NULL);
-    XtVaGetValues(XtParent(l->view), XtNwidth, &width,
+    XtVaGetValues(XtParent(XtParent(l->view)), XtNwidth, &width,
 		  XtNheight, &height,
-		  XtNhSpace, &hpad,
-		  XtNvSpace, &vpad,
 		  NULL);
-    width -= (hpad + bw) * 2;
-    height -= (vpad + bw) * 2;
+
+    width -= (bw + 2) * 2;
+    height -= bw * 2;
 
     /* XXX -- this really should be SetValues(width,height)
     **	      but that doesn't work..?/
      */
-    XtResizeWidget(l->view, width, height, 1);
+    XtMoveWidget(XtParent(l->view), 3, 30);
+    XtResizeWidget(XtParent(l->view), 
+                            (width>0)?width:1, 
+                            (height>0)?height:1, 0);
+    XtResizeWidget(l->view, (width>10)?width-10:1, 
+                            (height>38)?height-38:1, 1);
 
     resizeCursor(l);
 }
@@ -710,13 +716,13 @@ void
 FatbitsEdit(Widget paint)
 {
     Widget shell, form, fat;
-    Widget button, box;
+    Widget button, box, prev=None, *zoombut;
     int i, zoom;
     Colormap cmap;
     Position x, y, lx, ly;
     Dimension width = 48, height = 48;
     LocalInfo *l;
-    Arg args[3];
+    Arg args[4];
     int nargs = 0;
     static char *zoomList[] =
     {
@@ -728,7 +734,6 @@ FatbitsEdit(Widget paint)
     };
     XtTranslations trans =
     XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
-    Widget first = None;
 
     /* If a popup for this paint widget exists, raise it */
     for (l = head; l != NULL && l->paint != paint; l = l->next);
@@ -758,12 +763,48 @@ FatbitsEdit(Widget paint)
     PaletteAddUser(PaletteFind(shell, cmap), shell);
     form = XtVaCreateManagedWidget("form", formWidgetClass, shell, NULL);
 
+    button = XtVaCreateManagedWidget("done", commandWidgetClass, form,
+				     /*
+				     XtNfromVert, box,
+				     XtNtop, XtChainBottom,
+				     XtNbottom, XtChainBottom,
+				     XtNleft, XtChainLeft,
+				     XtNright, XtChainLeft,
+				     */
+				     NULL);
+
+    XtAddCallback(button, XtNcallback, doneCallback, (XtPointer) l);
+
+    zoombut = (Widget *) malloc(XtNumber(zoomList) * sizeof(Widget));
+    for (i = 0; i < XtNumber(zoomList); i++) {
+	zoombut[i] = XtVaCreateManagedWidget(zoomList[i],
+					 toggleWidgetClass, form,
+					 XtNfromHoriz, (i)?prev:button,
+				         /*
+					 XtNfromVert, box,
+					 XtNtop, XtChainBottom,
+					 XtNbottom, XtChainBottom,
+					 XtNleft, XtChainRight,
+					 XtNright, XtChainRight,
+					 */
+					 XtNradioGroup, prev,
+					 XtNtranslations, trans,
+					 NULL);
+	prev = zoombut[i];
+	XtAddCallback(prev, XtNcallback, buttonCallback, (XtPointer) l);
+
+	if (i == XtNumber(zoomList) / 2) {
+	    XtVaSetValues(button, XtNstate, True, NULL);
+	}
+    }
+
     box = XtVaCreateManagedWidget("paintBox", boxWidgetClass, form,
 			  XtNbackgroundPixmap, GetBackgroundPixmap(form),
+			  XtNvertDistance, 30,
 			  NULL);
 
     XtAddEventHandler(box, StructureNotifyMask, False,
-		      (XtEventHandler) boxChanged, (XtPointer) l);
+		      (XtEventHandler) fatbitsResized, (XtPointer) l);
 
 
     fat = XtVaCreateManagedWidget("paint", paintWidgetClass, box,
@@ -776,47 +817,12 @@ FatbitsEdit(Widget paint)
 
     XtVaSetValues(fat, XtNwidth, width * zoom, XtNheight, height * zoom, NULL);
 
-    button = XtVaCreateManagedWidget("done", commandWidgetClass, form,
-				     XtNfromVert, box,
-				     /*
-				     XtNtop, XtChainBottom,
-				     XtNbottom, XtChainBottom,
-				     XtNleft, XtChainLeft,
-				     XtNright, XtChainLeft,
-				     */
-				     NULL);
-
-    XtAddCallback(button, XtNcallback, doneCallback, (XtPointer) l);
-
-    ccpAddStdPopup(fat, NULL);
-
-    first = None;
-    for (i = 0; i < XtNumber(zoomList); i++) {
-	button = XtVaCreateManagedWidget(zoomList[i],
-					 toggleWidgetClass, form,
-					 XtNfromHoriz, button,
-					 XtNfromVert, box,
-					 /*
-					 XtNtop, XtChainBottom,
-					 XtNbottom, XtChainBottom,
-					 XtNleft, XtChainRight,
-					 XtNright, XtChainRight,
-					 */
-					 XtNradioGroup, first,
-					 XtNtranslations, trans,
-					 NULL);
-	first = button;
-	XtAddCallback(button, XtNcallback, buttonCallback, (XtPointer) l);
-
-	if (i == XtNumber(zoomList) / 2) {
-	    XtVaSetValues(button, XtNstate, True, NULL);
-	}
-    }
 
     XtAddCallback(shell, XtNdestroyCallback, destroyCallback, (XtPointer) l);
     AddDestroyCallback(shell, (DestroyCallbackFunc) doneCallback, (XtPointer) l);
 
     XtAddCallback(fat, XtNdestroyCallback, FatCursorDestroyCallback, fat);
+    ccpAddStdPopup(fat, NULL);
 
     XtPopup(shell, XtGrabNone);
 
@@ -836,7 +842,8 @@ FatbitsEdit(Widget paint)
 				  XtNwidth, width + 2 * (PADDING + 1),
 				  XtNheight, height + 2 * (PADDING + 1),
 				  XtNborderWidth, 1,
-			 XtNbackgroundPixmap, GetBackgroundPixmap(paint),
+			          XtNbackgroundPixmap, 
+                                         GetBackgroundPixmap(paint),
 				  NULL);
 
     l->cursor = box;
@@ -874,5 +881,14 @@ FatbitsEdit(Widget paint)
     GraphicAdd(fat);
     StateAddParent(shell, paint);
 
+    XtUnmanageChild(button);
+    XMapWindow(XtDisplay(button), XtWindow(button));
+    XtUnmanageChild(box);
+    XMapWindow(XtDisplay(box), XtWindow(box));
+
+    for (i=0; i<XtNumber(zoomList); i++) {
+      XtUnmanageChild(zoombut[i]);
+      XMapWindow(XtDisplay(zoombut[i]), XtWindow(zoombut[i]));
+    }
     return;
 }
