@@ -308,9 +308,9 @@ emptyList(arg_t * arg)
 static void
 browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flg)
 {
-    Dimension width, height, width2;
+    Dimension width, height, width2, width3;
     Position x, y;
-    int i, width1, height1, dw = 0;
+    int i, j, width1, height1, dw = 0;
 
     XtVaGetValues(XtParent(l->parent), 
 		  XtNwidth, &width, XtNheight, &height, NULL);
@@ -319,7 +319,13 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flg)
     XtVaGetValues(l->ok, XtNwidth, &width2, NULL);
     i = width2;
     XtVaGetValues(l->cancel, XtNwidth, &width2, NULL);
-    i += width2 + 35;
+    i += width2 + 35; 
+    for (j=0; j<l->numf; j++) {
+        XtVaGetValues(l->format[j], XtNwidth, &width3, NULL);
+        width3 += 25;
+        if (width3>i) i = width3;
+    }
+
     width1 = (int)width - i; if (width1<60) width1 = 60;
     height1 = (int)height - 120; if (height1<60) height1 = 60;
 
@@ -830,14 +836,20 @@ buildOpenBrowser(Widget w, arg_t * arg)
     j = 0;
     k = 0;
     for (i = 0; list[i] != NULL; i++) {
-        while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i]))) {
-	    if (j<NUM_FORMATS) ++j;
-	}
-	name_format = index(msgText[IMAGE_FORMATS+j], ' ');
-	if (name_format) 
-	   ++name_format;
-	else
-	   name_format = list[i];
+        while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i])) &&
+	       j<NUM_FORMATS)
+	    ++j;
+        if (j == NUM_FORMATS) {
+ 	    fprintf(stderr, "Logic error: file-format string \"%s\" not found "
+ 	      "in msgText\n", list[i]);
+ 	    name_format = "[unknown]";
+ 	} else {
+ 	    name_format = index(msgText[IMAGE_FORMATS+j], ' ');
+ 	    if (name_format)
+ 	       ++name_format;
+ 	    else
+ 	       name_format = list[i];
+  	}
 	if (j<NUM_FORMATS) ++j;
 
 	toggle = XtVaCreateManagedWidget(list[i],
@@ -933,14 +945,21 @@ buildSaveBrowser(Widget w, arg_t * arg)
     j = 0;
     k = 0;
     for (i = 0; list[i] != NULL; i++) {
-        while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i]))) {
-	    if (j<NUM_FORMATS) ++j;
+        while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i]))
+	       && j<NUM_FORMATS) ++j;
+        if (j == NUM_FORMATS) {
+	    fprintf(stderr, 
+              "Logic error: file-format string \"%s\" not found in msgText\n", 
+	      list[i]);
+	    name_format = "[unknown]";
+	} else { 
+	    name_format = list[i];
 	}
 	name_format = index(msgText[IMAGE_FORMATS+j], ' ');
 	if (name_format) 
-	   ++name_format;
+	    ++name_format;
 	else
-	   name_format = list[i];
+	    name_format = list[i];
 	if (j<NUM_FORMATS) ++j;
 
 	toggle = XtVaCreateManagedWidget(list[i],
