@@ -13,7 +13,12 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: fillOp.c,v 1.11 1998/05/21 21:17:23 torsten Exp $ */
+/* $Id: fillOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtWindow XTWINDOW
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -60,7 +65,7 @@ static Boolean *mask;
 static int maskW, maskH;
 static int xMin, yMin, xMax, yMax;
 static int timeCount = 0;
-static int fillMode = 0, tfillMode = TFILL_RADIAL;
+static int fillMode = 0, gfillMode = GFILL_RADIAL;
 #if FEATURE_FRACTAL
 static int ffillMode = 0;
 #endif
@@ -439,7 +444,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	 */
 	if (l->type == 1)
 	    gradient = buildGradientPixmap(w, info, rect.width, rect.height,
-					   depth, tfillMode);
+					   depth, gfillMode);
 #ifdef FEATURE_FRACTAL
 	else
 	    gradient = buildFractalPixmap(w, info, rect.width, rect.height,
@@ -504,14 +509,14 @@ FillSetMode(int value)
 /*
  * Gradient fill
  */
-static float TFillQuant = 50.0;
-static double TFillVoffset = 0.0;
-static double TFillHoffset = 0.0;
-static double TFillPad = 0.0;
-static int TFillTilt = 0;
+static float GradientFillQuant = 300.0;
+static double GradientFillVoffset = 0.0;
+static double GradientFillHoffset = 0.0;
+static double GradientFillPad = 0.0;
+static int GradientFillTilt = 0;
 
 void *
-TFillAdd(Widget w)
+GradientFillAdd(Widget w)
 {
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     l->done = False;
@@ -525,7 +530,7 @@ TFillAdd(Widget w)
 }
 
 void 
-TFillRemove(Widget w, void *l)
+GradientFillRemove(Widget w, void *l)
 {
     OpRemoveEventHandler(w, opPixmap, ButtonPressMask, FALSE,
 			 (OpEventProc) press, l);
@@ -535,9 +540,9 @@ TFillRemove(Widget w, void *l)
 }
 
 void 
-TFillSetMode(int value)
+GradientFillSetMode(int value)
 {
-    tfillMode = value;
+    gfillMode = value;
 }
 
 /*
@@ -545,13 +550,13 @@ TFillSetMode(int value)
  * angle is in degrees
  */
 void 
-TfillSetParameters(float voffset, float hoffset, float pad, int angle, int steps)
+GradientFillSetParameters(float voffset, float hoffset, float pad, float angle, int steps)
 {
-    TFillVoffset = voffset;
-    TFillHoffset = hoffset;
-    TFillPad = pad;
-    TFillTilt = angle;
-    TFillQuant = steps;
+    GradientFillVoffset = voffset;
+    GradientFillHoffset = hoffset;
+    GradientFillPad = pad;
+    GradientFillTilt = angle;
+    GradientFillQuant = (float) steps;
 }
 
 /*
@@ -559,7 +564,7 @@ TfillSetParameters(float voffset, float hoffset, float pad, int angle, int steps
  */
 #ifdef FEATURE_FRACTAL
 void *
-FFillAdd(Widget w)
+FractalFillAdd(Widget w)
 {
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
     l->done = False;
@@ -573,7 +578,7 @@ FFillAdd(Widget w)
 }
 
 void 
-FFillRemove(Widget w, void *l)
+FractalFillRemove(Widget w, void *l)
 {
     OpRemoveEventHandler(w, opPixmap, ButtonPressMask, FALSE,
 			 (OpEventProc) press, l);
@@ -583,7 +588,7 @@ FFillRemove(Widget w, void *l)
 }
 
 void 
-FFillSetMode(int value)
+FractalFillSetMode(int value)
 {
     ffillMode = value;
 }
@@ -631,20 +636,20 @@ buildGradientPixmap(Widget w, OpInfo * info,
     gs = col->green / 256;
     bs = col->blue / 256;
 
-    if (TFillTilt < 0)
-	TFillTilt += 360;
+    if (GradientFillTilt < 0)
+	GradientFillTilt += 360;
 
     /* Some handy values */
-    ox = width * (TFillHoffset + 0.5);	/* Centre point */
-    oy = height * (0.5 - TFillVoffset);
-    cs = cos(TFillTilt * DEG2RAD);	/* For rotation of coords */
-    sn = sin(TFillTilt * DEG2RAD);
+    ox = width * (GradientFillHoffset + 0.5);	/* Centre point */
+    oy = height * (0.5 - GradientFillVoffset);
+    cs = cos(GradientFillTilt * DEG2RAD);	/* For rotation of coords */
+    sn = sin(GradientFillTilt * DEG2RAD);
     maxdist = MAX(ox, width - ox);	/* Max distance to any point in pixmap */
     maxdist = MAX(maxdist, oy);
     maxdist = MAX(maxdist, height - oy);
 
     switch (mode) {
-    case TFILL_RADIAL:
+    case GFILL_RADIAL:
 	/*
 	 * This mixes colours according to distance from the point
 	 * in question to a centre point.
@@ -653,10 +658,10 @@ buildGradientPixmap(Widget w, OpInfo * info,
 	    for (x = 0; x < width; x++) {
 		dx = x - ox;
 		dy = y - oy;
-		fraction = (sqrt(dx * dx + dy * dy) / maxdist - TFillPad) /
-		    (1.0 - 2 * TFillPad);
+		fraction = (sqrt(dx * dx + dy * dy) / maxdist - GradientFillPad) /
+		    (1.0 - 2 * GradientFillPad);
 		CLAMP(0.0, fraction, 1.0);
-		fraction = ((int) (fraction * TFillQuant)) / TFillQuant;
+		fraction = ((int) (fraction * GradientFillQuant)) / GradientFillQuant;
 		*op++ = (1 - fraction) * rp + fraction * rs;
 		*op++ = (1 - fraction) * gp + fraction * gs;
 		*op++ = (1 - fraction) * bp + fraction * bs;
@@ -664,7 +669,7 @@ buildGradientPixmap(Widget w, OpInfo * info,
 	}
 	break;
 
-    case TFILL_LINEAR:
+    case GFILL_LINEAR:
 	/*
 	 * This mixes colours according to distance from the point
 	 * in question to the x axis (in the transformed system).
@@ -674,17 +679,17 @@ buildGradientPixmap(Widget w, OpInfo * info,
 	 * (distance from the line y = ax+h to (w,0) or
 	 * from y = ax to (w,h), whichever is greater)
 	 */
-	if (TFillTilt == 90)	/* tan(90) goes towards infinity */
+	if (GradientFillTilt == 90)	/* tan(90) goes towards infinity */
 	    s = width;
 	else {
-	    a = tan(TFillTilt * DEG2RAD);
+	    a = tan(GradientFillTilt * DEG2RAD);
 	    s = abs(a * width + (a < 0 ? -height : height)) / sqrt(a * a + 1);
 	}
 	for (y = 0; y < height; y++)
 	    for (x = 0; x < width; x++) {
 		yy = x * sn + y * cs;	/* only transformed y coord is needed */
-		fraction = (((float) yy) / s - TFillPad) / (1.0 - 2 * TFillPad);
-		fraction = ((int) (fraction * TFillQuant)) / TFillQuant;
+		fraction = (((float) yy) / s - GradientFillPad) / (1.0 - 2 * GradientFillPad);
+		fraction = ((int) (fraction * GradientFillQuant)) / GradientFillQuant;
 		CLAMP(0.0, fraction, 1.0);
 		*op++ = (1 - fraction) * rp + fraction * rs;
 		*op++ = (1 - fraction) * gp + fraction * gs;
@@ -692,7 +697,7 @@ buildGradientPixmap(Widget w, OpInfo * info,
 	    }
 	break;
 
-    case TFILL_SQUARE:
+    case GFILL_SQUARE:
 	/*
 	 * This mixes colours according to distance from the point
 	 * in question to the axis farthest away (in the transformed system).
@@ -711,8 +716,8 @@ buildGradientPixmap(Widget w, OpInfo * info,
 		    s = yy;
 		else
 		    s = xx;
-		fraction = (((float) s) / maxdist - TFillPad) / (1.0 - 2 * TFillPad);
-		fraction = ((int) (fraction * TFillQuant)) / TFillQuant;
+		fraction = (((float) s) / maxdist - GradientFillPad) / (1.0 - 2 * GradientFillPad);
+		fraction = ((int) (fraction * GradientFillQuant)) / GradientFillQuant;
 		CLAMP(0.0, fraction, 1.0);
 		*op++ = (1 - fraction) * rp + fraction * rs;
 		*op++ = (1 - fraction) * gp + fraction * gs;
@@ -720,7 +725,7 @@ buildGradientPixmap(Widget w, OpInfo * info,
 	    }
 	break;
 
-    case TFILL_CONE:
+    case GFILL_CONE:
 	/*
 	 * Convert (x,y) to polar coordinates (r,a) and mix
 	 * colours according to the fraction a/360.
@@ -739,13 +744,13 @@ buildGradientPixmap(Widget w, OpInfo * info,
 		    while (a < 0)	/* normalize */
 			a += 360.0;
 		}
-		a += TFillTilt;
+		a += GradientFillTilt;
 		while (a > 360.0)
 		    a -= 360.0;
 		if (a > 180.0)	/* map (180,360) to (180,0) */
 		    a = 360.0 - a;
-		fraction = (a / 180.0 - TFillPad) / (1.0 - 2 * TFillPad);
-		fraction = ((int) (fraction * TFillQuant)) / TFillQuant;
+		fraction = (a / 180.0 - GradientFillPad) / (1.0 - 2 * GradientFillPad);
+		fraction = ((int) (fraction * GradientFillQuant)) / GradientFillQuant;
 		CLAMP(0.0, fraction, 1.0);
 		*op++ = (1 - fraction) * rp + fraction * rs;
 		*op++ = (1 - fraction) * gp + fraction * gs;
