@@ -12,22 +12,29 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: help.c,v 1.4 1996/04/19 07:59:10 torsten Exp $ */
+/* $Id: help.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <stdio.h>
+#include <ctype.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Label.h>
-#include <X11/Xaw/List.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Text.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Scrollbar.h>
 #include <X11/StringDefs.h>
-#include <stdio.h>
-#include <ctype.h>
+
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Label.h"
+#include "xaw_incdir/List.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Text.h"
+#ifdef XAW3D
+#include "X11/IntrinsicP.h"
+#include "xaw_incdir/TextP.h"
+#include "xaw_incdir/SmeThreeD.h"
+#endif
+#include "xaw_incdir/Viewport.h"
+#include "xaw_incdir/Scrollbar.h"
 
 #include "xpaint.h"
 #include "Paint.h"
@@ -398,7 +405,7 @@ buildPopup(LocalInfo * l, Widget parent)
 				       XtNright, XtChainRight,
 				       XtNfromVert, textTitle,
 				       XtNfromHoriz, topicPort,
-				  XtNscrollVertical, XawtextScrollAlways,
+				       XtNscrollVertical, XawtextScrollAlways,
 				       XtNdisplayCaret, False,
 				       NULL);
     {
@@ -473,13 +480,26 @@ buildPopup(LocalInfo * l, Widget parent)
 
     XawListChange(topicList, l->topics, 0, 0, True);
 
-    XtAddCallback(topicList, XtNcallback,
-		  (XtCallbackProc) topicCB, (XtPointer) l);
     XtAddCallback(pgDown, XtNcallback, (XtCallbackProc) downCB, (XtPointer) l);
     XtAddCallback(pgUp, XtNcallback, (XtCallbackProc) upCB, (XtPointer) l);
     XtAddCallback(next, XtNcallback, (XtCallbackProc) nextCB, (XtPointer) l);
     XtAddCallback(prev, XtNcallback, (XtCallbackProc) prevCB, (XtPointer) l);
 
+    XtAddCallback(topicList, XtNcallback,
+		  (XtCallbackProc) topicCB, (XtPointer) l);
+    XtAddEventHandler(topicList, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
+    XtAddEventHandler(textList, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
+#ifdef XAW3D
+    {
+    Pixel pix;
+    XtVaGetValues(form, XtNbackground, &pix, NULL);
+    XtVaSetValues(((TextWidget)textList)->text.vbar, XtNbackground,
+                  pix, NULL);
+    }
+#endif   
+
     XtAddCallback(done, XtNcallback, doneCB, (XtPointer) l);
     AddDestroyCallback(shell, (DestroyCallbackFunc) doneCB, (XtPointer) l);
 
