@@ -13,21 +13,24 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: iprocess.c,v 1.13 1998/08/21 16:30:03 torsten Exp $ */
+/* $Id: iprocess.c,v 1.18 2005/03/20 20:15:32 demailly Exp $ */
 
-#include <X11/Intrinsic.h>
-#include <X11/Shell.h>
-#include <X11/StringDefs.h>
-#include <X11/Xatom.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Paned.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 #include <dlfcn.h>
+
+#include <X11/Intrinsic.h>
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/Xatom.h>
+
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Paned.h"
+
 #include "Paint.h"
 #include "PaintP.h"
 #include "xpaint.h"
@@ -36,14 +39,21 @@
 #include "messages.h"
 #include "misc.h"
 #include "protocol.h"
+#include "graphic.h"
+
+#ifdef XAW3D
+#define BORDERWIDTH 1
+#else
+#define BORDERWIDTH 0
+#endif
 
 typedef struct {
-    Widget shell, name, program, cclog;
+    Widget shell, pane, name, program, cclog;
     int mode;
     char *scriptfile;
 } LocalInfo;
 
-#define BUFSIZE 1024
+#define BUFSIZE 65536
 
 static void *dl_filter = NULL;
 
@@ -212,12 +222,12 @@ normalize(Image * image)
 	count = image->width * image->height * 3;
     }
 
-    for (ip = sp, i = 0; i < count; i++, ip++)
+    for (ip = (char*) sp, i = 0; i < count; i++, ip++)
 	if (*ip > maxval)
 	    maxval = *ip;
     if (maxval == 0)
 	return;
-    for (ip = sp, i = 0; i < count; i++, ip++)
+    for (ip = (char*) sp, i = 0; i < count; i++, ip++)
 	*ip = ((int) *ip * 255) / maxval;
 }
 
@@ -1720,9 +1730,9 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
    }
 
    /* compile C script */
-   sprintf(cmd, "cd /tmp\ngcc -I%s -c %s.c -o %s.o\n"
+   sprintf(cmd, "cd /tmp\ngcc -I%s/include -c %s.c -o %s.o\n"
                 "gcc -shared -Wl,-soname,%s.so %s.o -o %s.so\n",
-	        SHAREDIR"/include",
+	        GetShareDir(),
                 tmp, tmp, strstr(tmp, "XPaint"), tmp, tmp);
    XtVaSetValues(info->cclog, XtNstring, cmd, NULL);
    pipe_command(cmd, NULL, CollectOutput, CollectOutput);
@@ -1850,7 +1860,6 @@ loadFileCallbackOK(Widget w, XtPointer wlArg, char *file)
    LocalInfo *info = (LocalInfo *) wlArg;
    static char *text;
    char buf[2048];
-   char *ptr;
    int l;
 
    if (!file || !*file) return;
@@ -1932,7 +1941,6 @@ saveFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 static void
 saveasFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 {
-   LocalInfo *info = (LocalInfo *) wlArg;
    char buf[256];
    *buf = '\0';
    if (getcwd(buf, 256)) strcat(buf, "/");
@@ -1957,22 +1965,63 @@ loadScriptCallback(Widget w, XtPointer wlArg, XtPointer junk)
       }
 
    Global.explore = True;
-   sprintf(directory, "%s/%s/", SHAREDIR"/c_scripts", 
+   sprintf(directory, "%s/c_scripts/%s/", GetShareDir(),
 	              predefMenu[info->mode].name);
    GetFileName(GetShell(w), 3, directory, 
       (XtCallbackProc) loadFileCallbackOK, wlArg);
    Global.explore = False;
 }
 
+void
+editorResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height;
+    Dimension width1, height1, height2;
+
+    XtVaGetValues(l->shell, XtNwidth, &width, XtNheight, &height, NULL);
+    XtResizeWidget(XtParent(l->name), width, 38, BORDERWIDTH);
+    XtResizeWidget(l->pane, width, height, BORDERWIDTH);
+    XtVaSetValues(l->pane, XtNwidth, width, XtNheight, height, NULL);
+    XtVaGetValues(XtParent(l->program), XtNheight, &height1, NULL);
+
+#ifdef XAW3DG   
+    width1 = (width>12)? width-10 : 2;
+    height1 = (height1>10)? height1-8 : 2;
+    height2 = (height>height1+63)? height - height1 - 53 : 10;
+#else
+    width1 = (width>10)? width-8 : 2;
+    height1 = (height1>10)? height1-8 : 2;
+    height2 = (height>height1+60)? height - height1 - 50 : 10;
+#endif   
+    XtResizeWidget(XtParent(l->program), width1+8, height1+8, BORDERWIDTH);
+    XtResizeWidget(l->program, width1, height1, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->program), 
+       XtNwidth, width1+8, XtNheight, height1+8, NULL);
+    XtVaSetValues(l->program, 
+       XtNwidth, width1, XtNheight, height1, NULL);
+    XtMoveWidget(XtParent(l->program), 0, 39);
+    XtMoveWidget(l->program, 4, 0);
+
+    XtResizeWidget(XtParent(l->cclog), width1+8, height2+8, BORDERWIDTH);
+    XtResizeWidget(l->cclog, width1, height2, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->cclog), 
+       XtNwidth, width1+8, XtNheight, height2+8, NULL);
+    XtVaSetValues(l->cclog, 
+       XtNwidth, width1, XtNheight, height2, NULL);
+    XtMoveWidget(XtParent(l->cclog), 0, 46+height1);
+    XtMoveWidget(l->cclog, 4, 0);
+}
+
 void
 ScriptEditor(Widget w, Widget paint)
 {
     Display *dpy;
     Window root;
-    Widget pane; 
     Widget topform, midform, botform;
     Widget bar, cancel, compile;
     LocalInfo *info = XtNew(LocalInfo);
+    Arg args[4];
+    int nargs = 0;
     int i;
 
     head = info;
@@ -1984,10 +2033,10 @@ ScriptEditor(Widget w, Widget paint)
 
     info->shell = XtVisCreatePopupShell("XPaint / C scripts", 
                                   topLevelShellWidgetClass,
-				  Global.toplevel, NULL, 0);
-    pane = XtVaCreateManagedWidget("pane", panedWidgetClass, info->shell,
+				  Global.toplevel, args, nargs);
+    info->pane = XtVaCreateManagedWidget("pane", panedWidgetClass, info->shell,
                                    NULL);
-    topform = XtVaCreateManagedWidget("topform", formWidgetClass, pane, 
+    topform = XtVaCreateManagedWidget("topform", formWidgetClass, info->pane, 
                                       NULL);
     bar = MenuBarCreate(topform, XtNumber(printMenuBar), printMenuBar);
     XtVaSetValues(bar, XtNvertDistance, 2, NULL);
@@ -1995,14 +2044,17 @@ ScriptEditor(Widget w, Widget paint)
     info->name = XtVaCreateManagedWidget("name",
 				    labelWidgetClass, topform,
 				    XtNlabel, "",
-				    XtNwidth, 250,
-                                    XtNhorizDistance, 100,
+				    XtNwidth, 350,
+				    XtNfromHoriz, bar,
+                                    XtNhorizDistance, 15,
                                     XtNvertDistance, 6,
+				    XtNborderWidth, 0,
                                     NULL);
 
     compile = XtVaCreateManagedWidget("compile",
 		 		    commandWidgetClass, topform,
-				    XtNhorizDistance, 355,
+				    XtNfromHoriz, bar,
+				    XtNhorizDistance, 400,
 				    XtNvertDistance, 5,
 				    NULL);
     cancel = XtVaCreateManagedWidget("close",
@@ -2012,8 +2064,10 @@ ScriptEditor(Widget w, Widget paint)
 				    XtNvertDistance, 5,
 				    NULL);
 
-    midform = XtVaCreateManagedWidget("midform", formWidgetClass, pane, NULL);
-    botform = XtVaCreateManagedWidget("botform", formWidgetClass, pane, NULL);
+    midform = XtVaCreateManagedWidget("midform", 
+                                    formWidgetClass, info->pane, NULL);
+    botform = XtVaCreateManagedWidget("botform", 
+                                    formWidgetClass, info->pane, NULL);
 
     info->program = 
           XtVaCreateManagedWidget("program", asciiTextWidgetClass, midform,
@@ -2059,11 +2113,31 @@ ScriptEditor(Widget w, Widget paint)
        XtAddCallback(predefMenu[i].widget, XtNcallback, 
 		     (XtCallbackProc) loadScriptCallback, (XtPointer) info);
 
-    XtRealizeWidget(info->shell);
+    /* XtRealizeWidget(info->shell); */
     XtPopup(info->shell, XtGrabNone);
     XtVaSetValues(topform, XtNshowGrip, False, NULL);
     XtVaSetValues(midform, XtNshowGrip, True, NULL);
     XtVaSetValues(botform, XtNshowGrip, True, NULL);
+
+    XtUnmanageChild(info->name);
+    XMapWindow(dpy, XtWindow(info->name));
+    XtUnmanageChild(cancel);
+    XMapWindow(dpy, XtWindow(cancel));
+    XtUnmanageChild(compile);
+    XMapWindow(dpy, XtWindow(compile));
+    XtUnmanageChild(info->program);
+    XMapWindow(dpy, XtWindow(info->program));
+    XtUnmanageChild(info->cclog);
+    XMapWindow(dpy, XtWindow(info->cclog));      
+   
+    XtAddEventHandler(midform, StructureNotifyMask, False,
+		      (XtEventHandler) editorResized, (XtPointer) info);
+    XtAddEventHandler(info->shell, StructureNotifyMask, False,
+		      (XtEventHandler) editorResized, (XtPointer) info);
+    XtAddEventHandler(info->program, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
+    XtAddEventHandler(info->cclog, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
 }
 
 /* convenience routines */
@@ -2112,9 +2186,7 @@ Image *
 ImageUserDefined(Image * input)
 {
     Image *output;
-    unsigned char *p, *op;
     char * error;
-    int x, y;
     void (* proc)(Image *, Image *);
     
     if (!dl_filter) return NULL;
