@@ -15,6 +15,8 @@
 
 /* $Id: main.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
+#define DEFAULT_GEOMETRY "640x480"
+
 #ifdef __VMS
 #define XtDisplay XTDISPLAY
 #define XtScreen XTSCREEN
@@ -70,6 +72,7 @@ typedef struct {
     String size;
     String winsize;
     String help;
+    String shareDir;   
     String rcFile;
     String helpFile;
     String msgFile;
@@ -92,9 +95,11 @@ static XtResource resources[] =
     {"lang", "Lang", XtRString, sizeof(String),
      XtOffset(AppInfo *, lang), XtRImmediate, (XtPointer) NULL},
     {"size", "Size", XtRString, sizeof(String),
-     XtOffset(AppInfo *, size), XtRImmediate, (XtPointer) "640x480"},
+     XtOffset(AppInfo *, size), XtRImmediate, (XtPointer) DEFAULT_GEOMETRY},
     {"winsize", "WinSize", XtRString, sizeof(String),
      XtOffset(AppInfo *, winsize), XtRImmediate, (XtPointer) "0x0"},
+    {"shareDir", "ShareDir", XtRString, sizeof(String),
+     XtOffset(AppInfo *, shareDir), XtRImmediate, (XtPointer) NULL},   
     {"rcFile", "RcFile", XtRString, sizeof(String),
      XtOffset(AppInfo *, rcFile), XtRImmediate, (XtPointer) NULL},
     {"help", "Help", XtRString, sizeof(String),
@@ -128,9 +133,10 @@ static XrmOptionDescRec options[] =
     {"-language", ".lang", XrmoptionSepArg, (XtPointer) NULL},
     {"-size", ".size", XrmoptionSepArg, (XtPointer) NULL},
     {"-winsize", ".winsize", XrmoptionSepArg, (XtPointer) "0x0"},
-    {"-rcFile", ".rcFile", XrmoptionSepArg, (XtPointer) NULL},
-    {"-helpFile", ".helpFile", XrmoptionSepArg, (XtPointer) NULL},
-    {"-msgFile", ".msgFile", XrmoptionSepArg, (XtPointer) NULL},
+    {"-sharedir", ".shareDir", XrmoptionSepArg, (XtPointer) SHAREDIR},   
+    {"-rcfile", ".rcFile", XrmoptionSepArg, (XtPointer) NULL},
+    {"-helpfile", ".helpFile", XrmoptionSepArg, (XtPointer) NULL},
+    {"-msgfile", ".msgFile", XrmoptionSepArg, (XtPointer) NULL},
     {"-zoom", ".zoom", XrmoptionSepArg, (XtPointer) "-1"},
     {"-simplepopup", ".fullpopup", XrmoptionNoArg, (XtPointer) "False"},
     {"-fullpopup", ".fullpopup", XrmoptionNoArg, (XtPointer) "True"},
@@ -230,6 +236,12 @@ GetDefaultRC(void)
     return appInfo.rcFile;
 }
 
+char *
+GetShareDir(void)
+{
+    return appInfo.shareDir;
+}
+
 /*
 **  Create a nice icon image for XPaint...
  */
@@ -395,15 +407,27 @@ LoadMessages()
 {
     FILE *fd;
     int i, j;
+    char namebuf[256];
     char buf[256];
     static char alert[] = "!! No messages found !!";
-    static char failure[] = "Memory allocation failure\n";
+    static const char failure[] = "Memory allocation failure\n";
+    static const char msgs_ver_str[] = "# XPaint Messages version ";
+    static const int msgs_ver_str_len = sizeof(msgs_ver_str)/sizeof(char) - 1;
 
     helpNum = 0; 
-    fd = fopen(appInfo.helpFile, "r");
+
+    if (appInfo.helpFile) {
+       if (*appInfo.helpFile!='/' && *appInfo.helpFile!='.')
+          sprintf(namebuf, "%s/%s", appInfo.shareDir, appInfo.helpFile);
+       else
+	  strcpy(namebuf, appInfo.helpFile);
+       fd = fopen(namebuf, "r");
+    } else
+       fd = NULL;
+   
     if (!fd) {
-        sprintf(buf, "%s/help/Help", SHAREDIR);
-	fd = fopen(buf, "r");
+        sprintf(namebuf, "%s/help/Help", appInfo.shareDir);
+	fd = fopen(namebuf, "r");
     }
     if (fd) {
         while (!feof(fd)) {
@@ -419,19 +443,51 @@ LoadMessages()
         fprintf(stderr, "No help file found !!\n");
     }
 
-    msgNum = 0; 
-    fd = fopen(appInfo.msgFile, "r");
+    msgNum = 0;
+    if (appInfo.msgFile) {
+       if (*appInfo.msgFile!='/' && *appInfo.msgFile!='.')
+          sprintf(namebuf, "%s/%s", appInfo.shareDir, appInfo.msgFile);
+       else
+	  strcpy(namebuf, appInfo.msgFile);
+       fd = fopen(namebuf, "r");
+    } else
+       fd = NULL;
     if (!fd) {
-        sprintf(buf, "%s/messages/Messages", SHAREDIR);
-	fd = fopen(buf, "r");
+        sprintf(namebuf, "%s/messages/Messages", appInfo.shareDir);
+	fd = fopen(namebuf, "r");
     }
+
     if (fd) {
+        int version_checked = 0;
+       
         while (!feof(fd)) {
             fgets(buf, 250, fd);
 	    if (*buf == '#') {
-	         if (!strncmp(buf+1, "END_MESSAGES", 12)) break;
-	         continue;
+	        /* GRR 20050806:  Avoid version incompatibilities by requiring
+	         *  version number of runtime Messages file to match that with
+	         *  which we were compiled.  Note that it need not necessarily
+	         *  match XPAINT_VERSION, however. */
+	        if (!strncmp(buf, msgs_ver_str, msgs_ver_str_len)) {
+	            char *ptr = buf + msgs_ver_str_len;
+	            int last = strlen(ptr) - 1;
+
+	            if    (last >= 0 && ptr[last] == '\n')  ptr[last--] = '\0';
+	            while (last >= 0 && isspace(ptr[last])) ptr[last--] = '\0';
+	            if (strcmp(ptr, MESSAGES_VERSION)) {
+	                fprintf(stderr, "Error: %s is version %s; expected "
+	                  "version %s\n", namebuf, ptr, MESSAGES_VERSION);
+	                exit(1);
+	            }
+	            version_checked = 1;
+	        }
+	        if (!strncmp(buf+1, "END_MESSAGES", 12)) break;
+	        continue;
 	    }
+	    if (!version_checked) {
+	        fprintf(stderr, "Error: %s is version 2.7.7 or earlier; "
+	          "expected version %s\n", namebuf, MESSAGES_VERSION);
+	        exit(1);
+  	    }
             ++msgNum;
             msgText = (String *)realloc(msgText, msgNum*sizeof(String));
 	    if (!msgText) {
@@ -503,7 +559,8 @@ checkRCsize()
     char buf[256];
     char *ptr;
     int w, h, i;
-    if (!strcmp(appInfo.size, "640x480")) {
+
+    if (!strcmp(appInfo.size, DEFAULT_GEOMETRY)) {
         if (appInfo.rcFile)
 	    strcpy(buf, appInfo.rcFile);
 	else
@@ -572,7 +629,7 @@ main(int argc, char *argv[])
     app_def_file = GetAppDefaultFile();
     if (!app_def_file) {
         fprintf(stderr, "%s : %s\n",
-            msgText[CANT_FIND_APP_DEFAULTS_FILE], app_def_file);
+            "Can't find app_defaults file", app_def_file);
         exit(1);
     }
 
@@ -732,6 +789,7 @@ main(int argc, char *argv[])
     Global.nbrushes = 0;
     Global.brushes = NULL;
     Global.join = JoinMiter;
+    Global.escape = False;
     Global.dashlist[0] = '\0';
     Global.dashnumber = 0;
     Global.dashoffset = 0;
@@ -816,7 +874,7 @@ main(int argc, char *argv[])
      */
     sprintf(xpTitle, "XPaint %s", XPAINT_VERSION);
     XtVaSetValues(toplevel, XtNtitle, xpTitle, NULL);
- /* XtVaSetValues(toplevel, XtNiconName, xpTitle, XtNtitle, xpTitle, NULL); */
+    XtVaSetValues(toplevel, XtNiconName, xpTitle, XtNtitle, xpTitle, NULL);
 
     if (appInfo.snapshot) {
         SnapshotImage(toplevel, NULL, 0);
