@@ -13,18 +13,28 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: misc.c,v 1.12 1997/09/03 19:31:07 torsten Exp $ */
+/* $Id: misc.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
-#include <X11/cursorfont.h>
-#include <X11/Xaw/Label.h>
-#include <X11/Shell.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
 
+#include <X11/Intrinsic.h>
+#include <X11/StringDefs.h>
+#include <X11/cursorfont.h>
+#include <X11/Shell.h>
+#include <X11/IntrinsicP.h>
+
+#include "xaw_incdir/Label.h"
+#include "xaw_incdir/Scrollbar.h"
+#include "xaw_incdir/ScrollbarP.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Text.h"
+#include "xaw_incdir/TextP.h"
+#include "xaw_incdir/Viewport.h"
+#include "xaw_incdir/ViewportP.h"
+
 #include "bitmaps/xbm/background.xbm"
 
 #ifndef XtNcursor
@@ -38,6 +48,31 @@
 #include "palette.h"
 #include "protocol.h"
 
+/* 
+ *  X error handler for,  mainly for future use
+ */
+
+int 
+privateXErrorHandler(Display *dpy, XErrorEvent *myerr)
+{
+    char msg[82];  
+
+    /* if we get a BadAlloc error, it's probably 'cuz we couldn't allocate
+     * the pixmap.  Tell the user this.
+     */
+    if (myerr->error_code == None) return 0;
+    fprintf(stderr,"%s:\n", routine); 
+    if (myerr->error_code == BadAlloc) {
+        fprintf(stderr, msgText[INSUFFICIENT_RESOURCES_FOR_OPERATION]);
+	fprintf(stderr, msgText[PROBABLY_NOT_ENOUGH_MEMORY_TO_ALLOCATE_PIXMAP]);
+    }
+
+    /* Now print out the error string that X generates */
+    
+    XGetErrorText(dpy, myerr->error_code, msg, 80);
+    fprintf(stderr, "%s %s\n", msgText[ERROR_CODE], msg);
+    return 0;
+}
 
 Widget
 GetToplevel(Widget w)
@@ -544,3 +579,116 @@ XtVisCreatePopupShell(String name, WidgetClass widget_class,
 
     return XtCreatePopupShell(name, widget_class, parent, args, nargs);    
 }
+
+
+static Boolean CompareEvents (oldEvent, newEvent)
+    XEvent *oldEvent, *newEvent;
+{
+#define Check(field) if (newEvent->field != oldEvent->field) return False;
+
+    Check(xany.display);
+    Check(xany.type);
+    Check(xany.window);
+
+    switch (newEvent->type) {
+    case MotionNotify:
+        Check(xmotion.state);
+        break;
+    case ButtonPress:
+    case ButtonRelease:
+        Check(xbutton.state);
+        Check(xbutton.button);
+        break;
+    case KeyPress:
+    case KeyRelease:
+        Check(xkey.state);
+        Check(xkey.keycode);
+        break;
+    case EnterNotify:
+    case LeaveNotify:
+        Check(xcrossing.mode);
+        Check(xcrossing.detail);
+        Check(xcrossing.state);
+        break;
+    }
+#undef Check
+
+    return True;
+}
+
+struct EventData {
+    XEvent *oldEvent;
+    int count;
+};
+
+static Bool PeekNotifyEvent (dpy, event, args)
+    Display *dpy;
+    XEvent *event;
+    char *args;
+{
+    struct EventData *eventData = (struct EventData*)args;
+    
+    return ((++eventData->count == QLength(dpy)) /* since PeekIf blocks */
+            || CompareEvents(event, eventData->oldEvent));
+}
+
+static Boolean LookAhead (w, event)
+    Widget w;
+    XEvent *event;
+{
+    XEvent newEvent;
+    struct EventData eventData;
+
+    if (QLength (XtDisplay (w)) == 0) return False;
+    
+    eventData.count = 0;
+    eventData.oldEvent = event;
+                        
+    XPeekIfEvent (XtDisplay (w), &newEvent, PeekNotifyEvent, 
+                  (char*)&eventData);
+                        
+    return CompareEvents (event, &newEvent);
+}
+
+void
+mousewheelScroll(Widget w, void * l, XEvent * event, Boolean * flg)
+{
+    Widget sb;
+    int call_data;
+    if (event->xbutton.button == 4 || event->xbutton.button == 5) {
+        if (l==NULL) {
+	    sb = ((TextWidget)w)->text.vbar;
+	}
+        else
+        if (l==(void *)1) {
+	    w = XtParent(w);
+	    sb = XtNameToWidget(w, "vertical");	   
+        } 
+        else
+        if (l==(void *)2) {
+	    w = XtParent(XtParent(w));
+	    sb = XtNameToWidget(w, "vertical");	   
+	}
+        else return;
+        if (sb == None) return;
+#ifdef XAW3DG
+        if (((ScrollbarWidget)sb)->scrollbar.scroll_mode == 2 /* if scroll continuous */
+            || LookAhead (sb, event))
+            return;
+#else
+        if (LookAhead (sb, event))
+	    return;
+#endif
+        if (((ScrollbarWidget)sb)->scrollbar.shown >= 1.0)
+            return; 
+#ifdef XAW95       
+        call_data = 4 * ((ScrollbarWidget)sb)->scrollbar.length / 2000;
+#else
+        call_data = 6 * ((ScrollbarWidget)sb)->scrollbar.length / 20;
+	if (call_data < 5) call_data = 5;       
+#endif       
+        if (event->xbutton.button == 4) call_data = -call_data;
+        XtCallCallbacks (sb, XtNscrollProc, (XtPointer)(call_data));
+    }
+}
+
