@@ -1471,6 +1471,9 @@ OperationInit(Widget toplevel)
     XtTranslations trans = XtParseTranslationTable(defTrans);
     IconListItem *cur;
     XpmAttributes attributes;
+#ifdef XAW3D
+    Pixel bg;
+#endif
 
     form = XtVaCreateManagedWidget("toolbox",
 				   formWidgetClass, toplevel,
@@ -1512,10 +1515,10 @@ OperationInit(Widget toplevel)
 				    XtNvertDistance, 5,
 #endif				    
 				    NULL);
+
     box = XtVaCreateManagedWidget("box",
 				    boxWidgetClass, vport,
 #ifdef XAW3D
-                                    XtNbackground, 0xc0c0c0,
  			            XtNwidth, 600, XtNheight, 600,
 #endif				  
 				    XtNtop, XtChainTop,
@@ -1586,7 +1589,7 @@ OperationInit(Widget toplevel)
 #else
         XtVaSetValues(icon, XtNwidth, 36, XtNheight, 32, XtNborderWidth, 0,
 #endif
-                            XtNforeground, 0xc0c0c0, NULL);
+                            NULL);
 #endif				       
 #if defined(XAW3D) && defined(XAW3DG)
 #ifdef BIGTOOLICONS
@@ -1594,9 +1597,12 @@ OperationInit(Widget toplevel)
 #else
         XtVaSetValues(icon, XtNwidth, 32, XtNheight, 32, XtNborderWidth, 0,
 #endif
-                            XtNforeground, 0xc0c0c0, NULL);
+                            NULL);
 #endif				       
-
+#ifdef XAW3D
+        XtVaGetValues(icon, XtNbackground, &bg, NULL);
+        XtVaSetValues(icon, XtNforeground, bg, NULL);
+#endif
 	if (cur->nitems != 0 && cur->popupMenu != NULL)
 	    MenuPopupCreate(icon, "popup-menu", cur->nitems, cur->popupMenu);
     }
