@@ -1,7 +1,7 @@
-/* $Id: operation.h,v 1.1 1996/02/07 10:32:18 torsten Exp $ */
+/* $Id: operation.h,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 /* operation.c */
-void OperationSelectCallAcross(int n);
+void OperationSelectCall(int n);
 void OperationSet(String names[], int num);
 void OperationSetPaint(Widget paint);
 void OperationAddArg(Arg arg);
