@@ -12,7 +12,12 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: palette.c,v 1.3 1996/04/19 08:53:26 torsten Exp $ */
+/* $Id: palette.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtScreen XTSCREEN
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -146,13 +151,13 @@ paletteNew(Widget w, Boolean useDefault)
 	    map->bRange <<= 1;
     case StaticGray:
     case StaticColor:
-	map->readonly = True;
+	map->readonly_ = True;
 	map->cmap = XCreateColormap(dpy, RootWindowOfScreen(screen),
 				    visual, AllocNone);
 	map->isDefault = False;
 	goto addlist;
     default:
-	map->readonly = False;
+	map->readonly_ = False;
 	if (useDefault) {
 	    map->cmap = rcmap;
 	    map->isDefault = True;
@@ -323,7 +328,7 @@ addColor(Palette * map, XColor * color)
     int i;
 
 
-    if (map->readonly) {
+    if (map->readonly_) {
 	/*
 	**  The temporary color is needed since alloc
 	**  will change the pixel values.
@@ -372,7 +377,7 @@ addColor(Palette * map, XColor * color)
     node->color.green = color->green & 0xff00;
     node->color.blue = color->blue & 0xff00;
     node->color.flags = DoRed | DoGreen | DoBlue;
-    if (!map->readonly)
+    if (!map->readonly_)
 	XStoreColor(map->display, map->cmap, &node->color);
     HashAdd(map->htable, HASH(*color), node);
     HashAdd(map->ltable, HASH_PIXEL(color->pixel), node);
@@ -601,7 +606,7 @@ PaletteSetInvalid(Palette * map, Pixel pix)
 {
     Col col, *c;
 
-    if (!map->isMapped || map->readonly)
+    if (!map->isMapped || map->readonly_)
 	return;
 
     col.color.pixel = pix;
@@ -620,7 +625,7 @@ PaletteSetPixel(Palette * map, Pixel pixel, XColor * xcol)
 {
     Col col, *c;
 
-    if (map->readonly)
+    if (map->readonly_)
 	return False;
 
     if (map->isDefault)
