@@ -12,7 +12,12 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: pencilOp.c,v 1.3 1996/04/19 08:53:17 torsten Exp $ */
+/* $Id: pencilOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtWindow XTWINDOW
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -34,6 +39,8 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     XRectangle undo;
     int save;
 
+    if (event->button >= Button4) return;   
+   
     if (info->surface == opWindow)
 	return;
 
@@ -135,7 +142,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     }
     if ((event->state & mask) != 0)
 	return;
-    if (event->button == Button3) return;
+    if (event->button >= Button3) return;
 
     if (l->isDots) {
 	XFreeGC(XtDisplay(w), l->gc);
