@@ -13,7 +13,15 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: protocol.c,v 1.4 1996/05/24 07:33:03 torsten Exp $ */
+/* $Id: protocol.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtIsRealized XTISREALIZED
+#define XtParent XTPARENT
+#define XtScreen XTSCREEN
+#define XtWindow XTWINDOW
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -21,6 +29,7 @@
 #include <X11/cursorfont.h>
 #include <sys/time.h>
 #include <stdio.h>
+#include <stdlib.h>
 
 #include "xpaint.h"
 #include "messages.h"
