@@ -19,7 +19,7 @@
 #include <sys/param.h>
 #endif
 
-#if defined(SVR2) || defined(SVR3) || defined(SVR4)
+#if defined(SVR2) || defined(SVR3) || defined(SVR4) || defined(linux)
 #ifndef SYSV
 #define SYSV
 #endif
