@@ -20,6 +20,8 @@
 #include "image.h"
 #include "rwTable.h"
 
+extern char * GetShareDir();
+
 int
 TestScriptC(char *file)
 {
@@ -60,9 +62,9 @@ ReadScriptC(char *file)
     if (ptr) *ptr = '\0';
 
     /* compile C script */
-    sprintf(cmd, "gcc -I%s -c %s -o /tmp/%s.o ; "
+    sprintf(cmd, "gcc -I%s/include -c %s -o /tmp/%s.o ; "
                  "gcc -shared -Wl,-soname,%s.so /tmp/%s.o -o /tmp/%s.so\n",
-	    	 SHAREDIR"/include",
+	    	 GetShareDir(),
                  file, radix, radix, radix, radix);
     system(cmd);
 
