@@ -116,15 +116,11 @@ static char ico_magic_number[4] = {0, 0, 1, 0};
  * Conversion
  */
 
-/* The different possible number of colors */
-#define LOW_COLOR 16
-#define HIGH_COLOR 0
-
 /* The color index of a pixel in the icon colormap */
 #define PIXEL_INDEX(x,y) \
-     (color_level == LOW_COLOR \
-      ? ((x) % 2 == 0 \
-	 ? (image [bytes_per_line * (height - 1 - (y)) / 2 + (x) / 2] & 0xF0) >> 4 \
+   (color_level == 4 ? \
+     ((x) % 2 == 0 ? \
+      (image [bytes_per_line * (height - 1 - (y)) / 2 + (x) / 2] & 0xF0) >> 4 \
 	 : image [bytes_per_line * (height - 1 - (y)) / 2 + (x) / 2] & 0x0F) \
       : image [bytes_per_line * (height - 1 - (y)) + (x)])
      
@@ -132,12 +128,6 @@ static char ico_magic_number[4] = {0, 0, 1, 0};
 #define PIXEL_REDUCED_INDEX(x,y) \
      (reduced_colormap_index [PIXEL_INDEX((x),(y))])
   
-/* The RGB value of a colormap entry */
-#define INDEX_VALUE(index,c) (colormap[(c) + (index) * 4])
-#define INDEX_R_VALUE(index) (INDEX_VALUE((index), 2))
-#define INDEX_G_VALUE(index) (INDEX_VALUE((index), 1))
-#define INDEX_B_VALUE(index) (INDEX_VALUE((index), 0))
-  
 /* The transparency of a pixel */
 #define MASK_BYTE(x,y) (bytes_per_mask_line * (height - 1 - (y)) + (x) / 8)
 #define MASK_BIT(x,y) (7 - (x) % 8)
@@ -145,7 +135,7 @@ static char ico_magic_number[4] = {0, 0, 1, 0};
      (mask[MASK_BYTE((x),(y))] & (1 << MASK_BIT((x),(y))))
   
 Image *
- ReadICO (char *file)
+ReadICO (char *file)
 {
    Image *xpaint_image = NULL;
 
@@ -166,6 +156,7 @@ Image *
    int width = 0;
    int height = 0;
    unsigned char bytes_per_line;
+   unsigned char bytes_per_pixel;
    unsigned char bytes_per_mask_line;
    unsigned char *dp, *mp = NULL;
    int image_length;
@@ -173,9 +164,9 @@ Image *
    int i, x, y;
 
 /* Variables for the reduced colormap */
-   int color_used[256];
+   int *reduced_colormap_index = NULL;
+   int *color_used = NULL;
    int nb_color_used;
-   unsigned char reduced_colormap_index [256];
 
 /* Does the icon have transparent pixels ? */
    int have_transparent_pixels;
@@ -227,47 +218,55 @@ Image *
     width = iconheader.idEntries[whichimage].bWidth;
     height = iconheader.idEntries[whichimage].bHeight;
 
-    /* Determine the number of bytes defining a line of the icon. */
-    bytes_per_line = 4 * ((width + 3) / 4);
-    bytes_per_mask_line = 4 * ((width + 31) / 32);
-
     /* Let's surf on over to the bitmap image & read the BMIH. */
     fseek (ico_stream, iconheader.idEntries[whichimage].dwImageOffset,SEEK_SET);
     fread (&bitmapinfoheader, 1, sizeof(bitmapinfoheader), ico_stream);
 
-    /* For debugging
-      fprintf(stdout,"(%d) bmih: %d %d %d %d %d %d %d %d %d %d %d\n",
-        whichimage,
-        (int)bitmapinfoheader.biSize,
-        (int)bitmapinfoheader.biWidth,
-        (int)bitmapinfoheader.biHeight,
-        (int)bitmapinfoheader.biPlanes,
-        (int)bitmapinfoheader.biBitCount,
-        (int)bitmapinfoheader.biCompression,
-        (int)bitmapinfoheader.biSizeImage,
-        (int)bitmapinfoheader.biXPelsPerMeter,
-        (int)bitmapinfoheader.biYPelsPerMeter,
-        (int)bitmapinfoheader.biClrUsed,
-        (int)bitmapinfoheader.biClrImportant);
-    */
-    
     /* Read the number of colors.
      * TODO: add support for monochrome, 24-bit icons
      */
-    switch(bitmapinfoheader.biPlanes
-        * bitmapinfoheader.biBitCount) {
-      case 4: /* 2^4 = 16 */ color_level = LOW_COLOR; break;
-      case 8: /* 2^8 = 256 */ color_level = HIGH_COLOR; break;
-      default:
+    /*
+        fprintf(stderr,
+          "ReadICO: %s:\n   Number of colors in image "
+          "(%d planes, %d bpp)\n",
+          file,
+          (int)bitmapinfoheader.biPlanes,
+          (int)bitmapinfoheader.biBitCount);
+        fprintf(stderr, 
+          "width = %d, height = %d (bpl = %d)\n", width, height); 
+    */
+
+    color_level = bitmapinfoheader.biPlanes * bitmapinfoheader.biBitCount ;
+    if (color_level&3 || color_level<4 || color_level>32 || 
+        (color_level>8 && color_level&7)) {
         fprintf(stderr,
           "ReadICO: %s:\n   Unsupported number of colors in image! "
           "Skipping. (%d planes, %d bpp)\n",
           file,
           (int)bitmapinfoheader.biPlanes,
           (int)bitmapinfoheader.biBitCount);
-        continue;
+        return NULL;
     }
     
+    /* Determine the number of bytes defining a line of the icon. */
+    if (color_level>=16) {
+       bytes_per_pixel = color_level/8;
+       bytes_per_line =  4 * ((bytes_per_pixel * width * 8 + 31) / 32);
+       bytes_per_mask_line = 4 * ((width + 31) / 32);
+    } else {
+       bytes_per_line =  4 * ((width * 8 + 31) / 32);
+       bytes_per_pixel = 1;
+       bytes_per_mask_line = 4 * ((width + 31) / 32);
+    }
+    /*
+    if (color_level==TRUE_COLOR) {
+       bytes_per_mask_line = 0;
+    } else {
+       bytes_per_line =  4 * ((width + 3) / 4);
+       bytes_per_mask_line = 4 * ((width + 31) / 32);
+    }
+    */
+
     /* Read the colormap */
     if (bitmapinfoheader.biClrImportant != 0)
       ncolors = bitmapinfoheader.biClrImportant;
@@ -276,15 +275,19 @@ Image *
     else
       ncolors = 1 << (bitmapinfoheader.biPlanes*bitmapinfoheader.biBitCount);
 
+    if (ncolors>256) ncolors = 256;
+    if (color_level>=16) ncolors = 0;
+
     fseek (ico_stream, iconheader.idEntries[whichimage].dwImageOffset
       + bitmapinfoheader.biSize, SEEK_SET);
     fread (colormap, 1, ncolors * 4, ico_stream);
 
     /* Read the image */
-    if (color_level == LOW_COLOR)
+    if (color_level == 4)
         image_length = bytes_per_line * height / 2;
     else
         image_length = bytes_per_line * height;
+
     fread (image, 1, image_length, ico_stream);
 
     /* Read the mask */
@@ -292,6 +295,7 @@ Image *
     fread (mask, 1, mask_length, ico_stream);
 
     /* Read error / prematured EOF  occured ? */
+
     if (feof (ico_stream))
       fprintf (stderr, "ReadICO: %s: Premature end of file.\n", file);
     if (ferror (ico_stream))
@@ -303,13 +307,19 @@ Image *
       }
 
     /* Reduce the number of colors */
+    if (ncolors) {
+        color_used = (int *)xmalloc(ncolors*sizeof(int));
+        reduced_colormap_index = (int *)xmalloc(ncolors*sizeof(int));
+    }
+
     for (i = 0 ; i < ncolors ; i++)
       color_used [i] = 0;
-  
+
+    if (ncolors)  
     for (y = 0 ; y <height ; y++)
       for (x = 0 ; x < width ; x++)
         color_used [PIXEL_INDEX(x,y)] = 1;
-  
+
     nb_color_used = 0;
     for (i = 0 ; i < ncolors ; i++)
       if (color_used [i])
@@ -347,16 +357,25 @@ Image *
 	        *dp++ = 0;
 		*mp = True;
 	    } else {
-	        i = PIXEL_INDEX(x,y);
-	        *dp++ = INDEX_R_VALUE(i);
-	        *dp++ = INDEX_G_VALUE(i);
-	        *dp++ = INDEX_B_VALUE(i);
+	      if (color_level>=16) {
+                /* Fix it : this should not work for color_level == 16 ... */
+                i = bytes_per_line * (height-1-y) + bytes_per_pixel * x + 2;
+                *dp++ = image[i--];
+                *dp++ = image[i--];
+                *dp++ = image[i];
+	      } else {
+	        i = 4 * PIXEL_INDEX(x,y);
+	        *dp++ = colormap[i--];
+	        *dp++ = colormap[i--];
+	        *dp++ = colormap[i];
+	      }
 	    }
 	    mp++;
 	  }
       }
   }
   fclose (ico_stream);
+  free(color_used);
   return xpaint_image;
 }
 
