@@ -4,7 +4,7 @@
 /* +-------------------------------------------------------------------+ */
 
 /* maintainer can nuke this line if not using CVS: */
-/* $Id: readWritePNG.c,v 1.1 2000/09/02 22:09:06 torsten Exp $ */
+/* $Id: readWritePNG.c,v 1.17 2005/03/20 20:15:34 demailly Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -51,7 +51,7 @@ TestPNG(char *file)  /* gets called a LOT on the first image:  brushes? */
     fread(header, 1, 8, fp);
     fclose(fp);
 
-    return png_check_sig(header, 8);
+    return png_check_sig( (unsigned char*) header, 8);
 }
 
 
