@@ -12,7 +12,14 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: writePS.c,v 1.5 2000/10/02 20:53:00 torsten Exp $ */
+/* $Id: readWritePS.c,v 1.14 2005/03/20 20:15:34 demailly Exp $ */
+
+/* default ghostcript resolution
+ * replace by 72 if 180 is too much to wish for ...
+ */
+
+#define GS_COMMAND \
+  "gs -g2000x2000 -sDEVICE=ppmraw -r180 -q -dNOPAUSE -sOutputFile="
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -329,14 +336,34 @@ int WriteResizedPS(char *file, Image * image, void *data)
     return 0;
 }
 
+int
+TestPS(char *file)
+{
+    char header[8];
+    FILE *fp = fopen(file, "rb");   /* libpng requires ANSI; so do we */
+
+    if (!fp)
+        return 0;
+
+    fread(header, 1, 8, fp);
+    fclose(fp);
+
+    if (!strncasecmp(header, "%!PS", 4)) return 1;
+    if (!strncasecmp(header, "%PDF", 4)) return 2;
+    if (!strncasecmp(header, "%TEX", 4)) return 3;
+    if (!strncasecmp(header, "%LTX", 4)) return 4;
+    return 0;
+}
+
 Image *
 ReadPS(char *file)
 {
     char *tmp;
-    char buf[512];
+    char rad[260];   
+    char buf[2048];
     FILE *fp;
     int format;
-    int wth, hth, w, h, pnm_type, bytes_per_pixel;
+    int wth, hth, w, h, pnm_type, bytes_per_pixel, type_doc;
     int x, y, i, j;
     int top, bottom, left, right;
     int *lborder, *rborder;
@@ -344,21 +371,35 @@ ReadPS(char *file)
     xel *curel;
     xelval maxval;
     char filled, newchr;
-    char *data, *dp;
+    char *compil, *data, *dp;
     Image *image;
 
     fp = fopen(file, "rb");
     if (!fp) return NULL;
     fclose(fp);
-
+   
     fp = openTemp(&tmp);
     if (!fp) return NULL;
     fclose(fp);
 
-    sprintf(buf, 
-      "gs -g2000x2000 -sDEVICE=ppmraw -r72 -q -dNOPAUSE -sOutputFile=%s -- %s",
-      tmp, file);
+    type_doc = TestPS(file);
+    if (!type_doc) return; /* should not happen anyway ... */
 
+    strncpy(rad, file, 256),
+    rad[256] = '\0';
+    if ((dp=rindex(rad, '.'))) *dp = '\0';
+   
+    if (type_doc>=3) {
+       if (type_doc==3) compil = "tex"; 
+       if (type_doc==4) compil = "latex";
+       sprintf(buf, 
+          "%s \"\\\\nonstopmode\\\\input\" %s ; dvips %s.dvi -o %s.ps ",
+	  compil, file, rad, rad);
+          system(buf);
+          strcat(rad, ".ps");
+    } else
+       strcpy(rad, file);
+    sprintf(buf, GS_COMMAND"%s -- %s", tmp, rad);
     system(buf);
 
     pnm_init2(tmp);
@@ -496,34 +537,3 @@ int WritePDF(char *file, Image * image)
     removeTemp();
     return 0;
 }
-
-int
-TestPS(char *file)
-{
-    char header[8];
-    FILE *fp = fopen(file, "rb");   /* libpng requires ANSI; so do we */
-
-    if (!fp)
-        return 0;
-
-    fread(header, 1, 8, fp);
-    fclose(fp);
-
-    return !strncasecmp(header, "%!PS", 4);
-}
-
-int
-TestPDF(char *file)
-{
-    char header[8];
-    FILE *fp = fopen(file, "rb");   /* libpng requires ANSI; so do we */
-
-    if (!fp)
-        return 0;
-
-    fread(header, 1, 8, fp);
-    fclose(fp);
-
-    return !strncasecmp(header, "%PDF", 4);
-}
-
