@@ -95,6 +95,7 @@ extern int TestJPEG(char *);
 extern int TestScriptC(char *);
 extern int TestPS(char *);
 extern int TestPDF(char *);
+extern int TestTEX(char *);
 
 #define DEF_READ_ENTRY	0
 #define DEF_WRITE_ENTRY	1
@@ -118,7 +119,8 @@ static ImageTypes RWtable[] =
     {"ICO_Format",       ReadICO,   WriteICO,  TestICO },
     {"PPM_Format",       ReadPNM,   WritePNM,  TestPNM },
     {"PS_Format",        ReadPS,    WritePS,   TestPS  },
-    {"PDF_Format",       ReadPS,    WritePDF,  TestPDF },
+    {"PDF_Format",       ReadPS,    WritePDF,  TestPS  },
+    {"TEX_Format",       ReadPS,        NULL,  TestPS  },
 #ifdef HAVE_SGI
     {"SGI_Format",       ReadSGI,   WriteSGI,  TestSGI },
 #endif
