@@ -13,7 +13,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: xpaint.h,v 1.9 1998/01/22 20:38:31 torsten Exp $ */
+/* $Id: xpaint.h,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
@@ -45,6 +45,8 @@
 #define NULL 0
 #endif				/* AIXV3 */
 
+extern char *routine;
+
 typedef struct {
     int x, y;
     Pixmap sourcepix, pix, mask;
@@ -61,12 +63,13 @@ extern struct Global_s {
     XtAppContext appContext;
     Boolean timeToDie;
     Boolean explore;
+    Boolean transparent;
     Time currentTime;
     XVisualInfo vis;
-    int join;  /* line caps and joins */
+    int join, escape;  /* line caps and joins */
     int dashnumber, dashoffset; /* dash parameters */
     char dashlist[50];
-    Widget toplevel, back, canvas, curpaint, patternshell, brushpopup;
+    Widget toplevel, bar, back, canvas, curpaint, patternshell, brushpopup;
     int nbrushes;
     void ** brushes;
     void * patterninfo;
@@ -104,10 +107,16 @@ struct Global_s Global;
 	(Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)
 
 /* gradient fill modes */
-#define TFILL_RADIAL 0
-#define TFILL_LINEAR 1
-#define TFILL_CONE   2
-#define TFILL_SQUARE 3
+#define GFILL_RADIAL 0
+#define GFILL_LINEAR 1
+#define GFILL_CONE   2
+#define GFILL_SQUARE 3
+
+/*
+**  Value at which zoom starts putting in "white" lines
+**    between pixels
+ */
+#define ZOOM_THRESH 5
 
 #define XYtoRECT(x1,y1,x2,y2,rect)					\
 			(rect)->x = MIN(x1,x2); 			\
